# -*- coding: utf-8 -*-

import googsystray
import threading
import urllib
import urllib2
import gtk
import time
import sys
import shlex
import os
import subprocess
import webbrowser
import socket

import GMain

def dialog_response(widget, res, data=None):
	widget.destroy()


def error_dialog(str):
	diag = gtk.MessageDialog(None, 0, 
	                                           gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
	                                           "Error")
	diag.format_secondary_markup(str)
	diag.connect("response", dialog_response, None)
	diag.show_all()



def run_command(cmd):
	try:
		
		if os.name == "nt":
			subprocess.Popen(cmd)
		else:
			cmd_arr1 = shlex.split(cmd)
			subprocess.Popen(cmd_arr1)
	except OSError, e:
		error_dialog("Could not run command \'cmd\'")




class GBase:
	def check_notifications(self, data=None):
		return False
		
	def get_gconfig(self):
		return GMain._GConf.gconfig["global"]

	def get_gbackupconfig(self):
		return GMain._GConf.backupconfig["global"]

	def check_thr(self, data=None):
		# Yes, these are references, but check_data will never modify
		# the account, so it's OK

		timeout = float(self.timeout) / 1000.0
		self.thread_cond.acquire()
		while self.thread_run:
			self.check_data()
			if not self.thread_run:
				break
			self.thread_cond.wait(timeout)
			
		self.thread_cond.release()
		
	def stop_timer(self):
		if (self.thread != None):
			self.thread_run = False
			self.thread_cond.acquire()
			self.thread_cond.notify()
			self.thread_cond.release()
			self.thread.join()
			self.thread = None

	def start_timer(self):
		self.stop_timer()
		self.thread_cond.acquire()
		self.thread = threading.Thread(target=self.check_thr)
		self.thread_run = True
		self.thread.start()
		self.thread_cond.release()
		
	def urlopen2(self, account, url, params=None, headers=None):
		data = None
		retry_count = 5
		f = None
		
		while True:
			try:
				if headers == None:
					f = account["opener"].open(url, params)
				else:
					req = urllib2.Request(url, params, headers)
					f = account["opener"].open(req)
				data = f.read()
				break
			except urllib2.HTTPError, e:
				# HTTP errors, usually permissions or minor
				# google hiccups.
				googsystray.debug("Error accessing: " + url + " - " + str(e.code) + " - " + e.msg)
				sys.exc_clear()
				if e.code == 500: # Internal Server Error
					# We see this from Google on occasion.
					# It seems to resolve itself fairly
					# quickly.
					f = None
					req = None
					time.sleep(1.0)
					googsystray.debug(headers)
					googsystray.debug(params)
					continue
				data = None
				break
			except socket.timeout, e:
				googsystray.debug("Socket timeout!")
				# try again
			except urllib2.URLError, e:
				# IO errors.  Network down, typically.  
				# Sleep + count retry.
				print "Error accessing: " + url,
				print e
				retry_count -= 1
				if retry_count == 0:
					data = None
					break
				continue
			except Exception, e:
				print "Error accessing: " + url
				print e
				sys.exc_clear() 
				data = None
				break
				# We'll try again here
	
		if f is not None:
			# Hack.  Urllib2 leaks memory otherwise.
			if f.fp is not None and hasattr(f.fp, "_sock"):
				f.fp._sock.recv = None

			f.close()
		return data


		
	def _send_post_block(self, args):
		args["lock"].acquire()
		acct = args["account"]
		while True:
			try:
				if args["headers"] == None:
					f = acct["opener"].open(args["url"], args["params"])
				else:
					req = urllib2.Request(args["url"], args["params"],
						args["headers"])
					f = acct["opener"].open(req)
				args["data"] = f.read()
				break
			except urllib2.HTTPError, e:
				print "Error accessing: " + args["url"],
				print " - " + str(e.code) + " - " + e.msg
				sys.exc_clear()
				args["data"] = None
				break
			except socket.timeout, e:
				print "Socket timeout!"
				# try again
			except:
				print "Error accessing: " + args["url"],
				sys.exc_clear() 
				# We'll try again here
		args["event"].set()
		args["lock"].release()
	
	
	def send_post_block(self, account, url, params=None, headers=None):
		event = threading.Event()
		lock = threading.Lock()
		args=[{"url":url,
		       "params":params,
		       "event":event,
		       "lock":lock,
		       "headers":headers,
		       "account":account,
		       "fp":None}];
		args[0]["lock"].acquire()
		threading.Thread(target=self._send_post_block, args=args).start()
		args[0]["lock"].release()
		
		while not args[0]["event"].isSet():
			while gtk.events_pending():
				gtk.main_iteration(False)
			time.sleep(0.01)
		args[0]["lock"].acquire()
		return args[0]["data"]
		
	def _browser_open(self, url):
		if self.get_gconfig()["browser_action"] == googsystray.ONCLICK_OTHER:
			try:
				run_command(self.get_gconfig()["browser_cmd"] % url)
			except TypeError, e:
				run_command(self.get_gconfig()["browser_cmd"])
		else:
			print "Opening " + url
			webbrowser.open(url, self.get_gconfig()["browser_action"])

	def browser_open(self, url):
		threading.Thread(target=self._browser_open, args=[url]).start()
	

	def get_auth(self, account, service):
		if account["type"] == "Google":
			type = "GOOGLE"
		elif account["type"] == "Apps":
			type = "HOSTED"
		
		
		params = urllib.urlencode({'Email':account["username"],
					   'Passwd':account["password"],
					   'accountType':type,
					   'service':service})
		data = self.urlopen2(account, "https://www.google.com/accounts/ClientLogin", params)

		if data == None:
			return
		
		c1 = data.find("Auth=")
		if c1 == -1:
			print "No authorization!"
			# fixme this is a problem
			return
		c2 = data.find("\n", c1)
		if c2 == -1:
			auth = "GoogleLogin auth=" + data[c1+5:]
		else:
			auth = "GoogleLogin auth=" + data[c1+5:c2]
			
		self.auth[account["name"]] = auth

	def clean_label(self, label_str):
		return label_str.replace("&", "&amp;")

	def get_account_from_string(self, name):
		for i in GMain._accounts:
			if i["name"] == name:
				return i
		return None		

	def __init__ (self):
		self.timeout = googsystray.DEFAULT_TIMEOUT
		self.notification_timer = None
		self.invalidated = False
		self.in_check = False
		self.auth = {}
		
		self.thread = None
		self.thread_run = False
		self.thread_cond = threading.Condition()
		self.thread_lock = threading.RLock()
		



