# -*- coding: utf-8 -*-

import gtk, gobject
import calendar
import threading
import copy
import time

from xml.etree import ElementTree as ET
from GIcon import GIcon			


class GCal(GIcon):
	def draw_message(self, message):
		mvbox = self.popup_window.vbox
		
		vbox = gtk.VBox()
		mvbox.pack_start(vbox, False, False, 0)
		vbox.set_data("msg_id", message["id"])
		self.popup_messages.append(vbox)
		
		event = message["data"]

		frame = gtk.Frame()
		vbox1 = gtk.VBox()
		frame.add(vbox1)
		vbox.pack_end(frame, False, False, 3)

		hbox = gtk.HBox()
		label = gtk.Label()
		label.set_markup("<b>" + event["title"] + "</b>")
		hbox.pack_start(label, False, False, 5)
		label = gtk.Label(time.strftime("%c", time.localtime(event["start_time"])))
		hbox.pack_end(label, False, False, 5)
		vbox1.pack_start(hbox, False, False, 5)


		hbox = gtk.HBox()
		label = gtk.Label()
		label.set_markup("<b>" + _("Where:") + "</b>")
		hbox.pack_start(label, False, False, 5)
		if event["where"] is None:
			label = gtk.Label()
			label.set_markup("<i>" + _("No location provided") + "</i>")
		else:
			label = gtk.Label()
			label.set_markup(event["where"])
		hbox.pack_start(label, False, False, 5)
		vbox1.pack_start(hbox, False, False, 0)

		hbox = gtk.HBox()
		label = gtk.Label()
		label.set_markup("<b>" + _("Description:") + "</b>")
		hbox.pack_start(label, False, False, 5)
		if event["content"] is None:
			label = gtk.Label()
			label.set_markup("<i>" + _("No description provided") + "</i>")
		else:
			label = gtk.Label()
			label.set_markup(event["content"])
		hbox.pack_start(label, False, False, 5)
		vbox1.pack_start(hbox, False, False, 0)
		

	def dumpit(self, node, depth=0):
		for i in range(depth):
			print "\t",
		print "<" + node.tag.encode("utf-8"),
		for i in node.attrib:
			print " " + i + " = \"" + node.attrib[i].encode("utf-8") + "\"",
		print ">",
		if node.text != None:
			print node.text.encode("utf-8")
		else:
			print ""
		
		for i in node.getchildren():
			self.dumpit(i, depth+1)
	

	def check_events(self, account, cal, feedurl):
		sttime = time.gmtime()
		# We'll get a weeks worth of events.  
		endtime = time.gmtime(time.time() + (3600*24*7))
		
		req_times = time.strftime("&start-min=%Y-%m-%dT%H:%M:00-00:00", sttime)
		req_times += time.strftime("&start-max=%Y-%m-%dT%H:%M:00-00:00", endtime)
		req_times += time.strftime("&recurrence-expansion-start=%Y-%m-%dT%H:%M:00-00:00", sttime)
		req_times += time.strftime("&recurrence-expansion-end=%Y-%m-%dT%H:%M:00-00:00", endtime)
		
		headers = {"Authorization" : self.auth[account["name"]] }

		data = self.urlopen2(account, feedurl+"?ctz=UTC&singleevents=true" + req_times, headers=headers)
		if data == None:
			return True		
		node = ET.XML(data)
		
#		title = node.find("{http://www.w3.org/2005/Atom}title").text

		for entry in node.findall("{http://www.w3.org/2005/Atom}entry"):
			entry_id = entry.find("{http://www.w3.org/2005/Atom}id").text

			upd = entry.find("{http://www.w3.org/2005/Atom}updated")
			tmpstr = upd.text[:-5] + " UTC"
			updtime = calendar.timegm(time.strptime(tmpstr, "%Y-%m-%dT%H:%M:%S %Z"))

			if cal["entries"].get(entry_id) != None:
				if cal["entries"][entry_id]["updated_time"] < updtime:
					del cal["entries"][entry_id]

			when = entry.find("{http://schemas.google.com/g/2005}when")
			# fixme these are, I believe, events that cover a whole day, like a holiday. 
			# once we start doing real calendar support, we'll need these.
			if (when == None):
				continue
				
			# fixme also these :)
			if (when.find("{http://schemas.google.com/g/2005}reminder") == None):
				continue
				
			event_title = entry.find("{http://www.w3.org/2005/Atom}title").text
			event_content = entry.find("{http://www.w3.org/2005/Atom}content").text
			where = entry.find("{http://schemas.google.com/g/2005}where")
			if where.get("valueString") != None:
				event_where = where.attrib["valueString"]
			else:
				event_where = None

			start_time = when.attrib["startTime"]
			if (len(start_time) == 10):
				start_time += "T00:00:00 UTC"
			else:
				start_time = start_time[:-5] + " UTC"

			st_time = calendar.timegm(time.strptime(start_time, "%Y-%m-%dT%H:%M:%S %Z"))
			
			# No reason to worry about past events.
			if (st_time < time.time()):
				continue


			cal["entries"][entry_id] = { "title" : event_title, 
							 "updated_time" : updtime,
							 "content" : event_content,
							 "where" : event_where,
							 "start_time" : st_time,
							 "reminders": [] }
			
			for reminder in when.findall("{http://schemas.google.com/g/2005}reminder"):
				if (reminder.attrib["method"] == "alert"):
					cal["entries"][entry_id]["reminders"].append({"alertTime": st_time-int(reminder.attrib["minutes"])*60,
												  "hasReminded":False,"isActive":False})
		
		
	 
	def icon_check_data(self, account):
		if account["name"] not in self.auth or self.auth[account["name"]] == None:
			self.get_auth(account, 'cl')
			
		headers = {"Authorization" : self.auth[account["name"]] }
		data = self.urlopen2(account, 'https://www.google.com/calendar/feeds/default/', headers=headers)
		
		if data == None:
			return True
		node = ET.XML(data)
		
		self.cal_lock.acquire()
		
		for entry in node.findall("{http://www.w3.org/2005/Atom}entry"):
			title = entry.find("{http://www.w3.org/2005/Atom}title").text
			cal = None
			for i in self.calendars:
				if i["title"] == title and i["account"] == account["name"]:
					cal = i
					break
			if cal is None:
				cal = { "title": title, "account": account["name"], "last_checked" : 0, "last_updated" : 0, "entries" : {} }
				self.calendars.append(cal)

		
			upd = entry.find("{http://www.w3.org/2005/Atom}updated")
			tmpstr = upd.text[:-5] + " UTC"
			updtime = calendar.timegm(time.strptime(tmpstr, "%Y-%m-%dT%H:%M:%S %Z"))
			
			# Check once a day whether we need to or not -- this is important because
			# we only get a weeks worth of stuff.  I suppose we could do this less often
			
			if updtime > cal["last_updated"] or time.time() > (cal["last_checked"] + (3600*24)):
				cal["last_checked"] = time.time()
				cal["last_updated"] = updtime
				content = entry.find("{http://www.w3.org/2005/Atom}content")
				self.check_events(account, cal, content.attrib["src"])
				
		
		self.cal_lock.release()
		
		gobject.idle_add(self.check_calendar_once, None)
		return True
		
	def check_calendar_once(self, data=None):
		self.check_calendar(data)

	def check_calendar(self, dummy=None):
		if not self.am_used():
			return False
			
		# Make sure we get the calendars list OK
		if not self.cal_lock.acquire(0):
			return True
		new_messages = []		

		for cal in self.calendars:
			if (cal["entries"] == None):
				continue
			for entry in cal["entries"].items():
				for reminder in entry[1]["reminders"]:
					if reminder["alertTime"] < time.time() and entry[1]["start_time"] > time.time():
						new_messages.append(
							{ "id": entry[0],
							  "sub_id": "reminder",
							  "account": cal["account"],
							  "count": 1,
							  "title": _("Calendar Reminder"),
							  "summary": entry[1]["title"],
							  "icon": None,
							  "marked": False,
							  "data": copy.deepcopy(entry[1]),
							  "type": self.config, 
							  })
						break

		self.cal_lock.release()	
		while not self.thread_lock.acquire(0):
			if not self.thread_run:
				return True
			time.sleep(0.1)
		self.messages_unread = new_messages
		gobject.idle_add(self.update_icon, None)
		self.thread_lock.release()

		
		return True
		
	def icon_stop(self):
		if self.cal_check_timer != -1:
			gobject.source_remove(self.cal_check_timer)
		return
		
	def icon_start (self):
		if not self.am_used():
			return
		self.cal_check_timer = gobject.timeout_add_seconds(60, self.check_calendar, None)
		return


				
	def __init__(self):
		self.config = "gcal"
		self.auth = {}
		
		self.cal_lock = threading.Lock()
	
		self.cal_check_timer = -1
		self.calendars = []
		
		GIcon.__init__(self)
