# -*- coding: utf-8 -*-

import googsystray
import gtk, gobject
import calendar
import threading
import copy
import os
import time

from xml.etree import ElementTree as ET
import GIcon			
import GMain

class GDocs(GIcon.GIcon):
	def label_clicked(self, widget, event, message):
		account = self.get_account_from_string(message["account"])
		if account["type"] == "Apps":
			docs_url = "https://docs.google.com/a/" + account["domain"]
			ss_url = "https://spreadsheets.google.com/a/" + account["domain"]
		else:
			docs_url = "https://docs.google.com"
			ss_url = "https://spreadsheets.google.com"
		
		kind = message["data"]["kind"]
		
		if kind == "document":
			url = docs_url + "/Doc?docid="
		if kind == "spreadsheet" or kind == "form":
			url = ss_url + "/ccc?key="
		if kind == "presentation":
			url = docs_url + "/present/edit?id="
		if kind == "pdf":
			url = docs_url + "/leaf?id="
			
		url += message["data"]["id"]
		
		self.browser_open(url)
		self.thread_lock.acquire()
		for i in self.messages_unread:
			if i["id"] == message["id"]:
				i["marked"] = True
				break
		self.thread_lock.release()
		gobject.idle_add(self.update_icon, None)



	def draw_message(self, message):
		data = message["data"]
		mvbox = self.popup_window.vbox

		scaled_size = int(48 * GMain._GConf.gconfig["global"]["popup_size"])
		pb = gtk.gdk.pixbuf_new_from_file(message["icon"])
		pb2 = pb.scale_simple(scaled_size, scaled_size, gtk.gdk.INTERP_HYPER)
		image = gtk.image_new_from_pixbuf(pb2)
		

		hbox = gtk.HBox()
		hbox.set_data("msg_id", message["id"])
		self.popup_messages.append(hbox)

		mvbox.pack_start(hbox, False, False, 3)
		hbox.pack_start(image, False, False, 5)
		
		vbox2 = gtk.VBox()
		hbox.pack_start(vbox2, True, True, 0)
			
		hbox2 = gtk.HBox()
		vbox2.pack_start(hbox2, False, False, 0)

		label = GIcon.link_label()
		label.set_markup("<b>%(doc)s</b> %(title)s" % { "doc" : _("Document:"),
								"title" : data["title"]})
		label.connect("button_press_event", self.label_clicked, message)
		hbox2.pack_start(label, False, False, 0)
		
		label = gtk.Label()
		label.set_markup("<span color=\"#404040\">" + data["upd_time"] + "</span>")
		label.set_justify(gtk.JUSTIFY_RIGHT)
		hbox2.pack_end(label, False, False, 5)
		
		hbox2 = gtk.HBox()
		vbox2.pack_start(hbox2, False, False, 0)

		if len(data["labels"]) > 0:
			label = gtk.Label()
			markup_str = "<b>%(label)s</b> <span color=\"#202020\">" % { "label" : _("Labels:") }
			first = True
			for i in data["labels"]:
				if first:
					first = False
					markup_str += i
					continue
				markup_str += ", " + i
			markup_str += "</span>"
			label.set_markup(markup_str)
			hbox2.pack_start(label, False, False, 0)	
		
		if self.num_used() > 1:
			label = gtk.Label()
			label.set_markup("<span color=\"#404040\"><i>(%(account)s)</i></span>" % {"account":message["account"]})
		
			hbox2.pack_end(label, False, False, 5)	

		
		
	def dumpit(self, node, depth=0):
		for i in range(depth):
			print "\t",
		print "<" + node.tag.encode("utf-8"),
		for i in node.attrib:
			print " " + i + " = \"" + node.attrib[i].encode("utf-8") + "\"",
		print ">",
		if node.text != None:
			print node.text.encode("utf-8")
		else:
			print ""
		
		for i in node.getchildren():
			self.dumpit(i, depth+1)
	

	 
	def icon_check_data(self, account):
		new_messages = []

		# Not being able to use the API sucks.  But the API 
		# is broken for the one thing I really need, which is the
		# "viewed" status.  So we've got to scrape the page.
		
		if account["type"] == "Apps":
			base_url = "https://docs.google.com/a/" + account["domain"] + "/"
		else:
			base_url = "https://docs.google.com/"
		
		data = self.urlopen2(account, base_url + "?browserok=true")

		if data is None:
			self.merge_messages(account["name"], new_messages)
			return

		fstr = "viewManagerEl.innerHTML = "
		c1 = data.find(fstr)
		c2 = data.find("\n", c1)
		
		html = eval(data[c1+len(fstr):c2-1])
		html = self.xmlize_html(html)
		node = ET.XML(html)
		tree = ET.ElementTree(node)
		walkAll = tree.getiterator()
		for i in walkAll:
			if i.tag == "tr":
				pdf = False
				cl = i.get("class")
				if cl is None:
					continue
				classes = cl.split(" ")
				if not 'doclist-tr' in classes:
					continue
				if 'doclist-tr-viewed' in classes:
					continue
				
				id_str = i.get("id")
				tmp = id_str.split(".")
				id = tmp[len(tmp)-1]
				labels = []
				
				for j in i.getiterator():
					if j.get("class") is None:
						continue
					if "documentpill-label" in j.get("class").split(" "):
						n = j;
						while n.text is None:
							n = n.getchildren()[0]
						labels.append(n.text)	
					if j.tag == "span" and j.get("class") is not None:
						if j.get("class") == "doclist-name":
							entry_title = j.get("title")
						elif j.get("class") == "doclist-lastedited":
							updtime = j.text
					if j.tag == "span" and j.get("style") is not None:
						if j.get("style").find("pdf") != -1:
							pdf = True
				
				if tmp[2] == "doc":
					kind = "document"
					icon_name = "application-document.png"
				elif tmp[2] == "spread":
					kind = "spreadsheet"
					icon_name = "application-spreadsheet.png"
				elif tmp[2] == "form":
					kind = "form"
					icon_name = "application-form.png"
				elif tmp[2] == "pres":
					kind = "presentation"
					icon_name = "application-presentation.png"
				elif tmp[2] == "DoclistBlob":
					kind = "pdf"
					icon_name = "application-pdf.png"
					# fixme, use the 'pdf' variable to seperate
					# generic data
				else:
					kind = "document"
					icon_name = "application-document.png"
					print "Unknown: " + tmp[2]
					
				icon = os.path.join(googsystray.BASE_DIR, "icons", icon_name)
			
				new_messages.append({ 
					"id": id + account["name"],
					"sub_id": "doc",
					"account": account["name"],
					"count": 1,
					"title": _("Google Document Updated"),
					"summary": entry_title,
					"icon": icon,
					"marked": False,
					"data": { 
						"kind" : kind,
						"upd_time" : updtime,
						"title" : entry_title,
						"labels" : labels,
						"id" : id,
					},
				})
			
		
	
	
		"""
		#	The following code _SHOULD_ work.  But I think the 
		#	Google Docs API is broken on showing what's viewed
		if account["name"] not in self.auth or self.auth[account["name"]] == None:
			self.get_auth(account, 'writely')
			
		headers = {"Authorization" : self.auth[account["name"]],
			   "GData-Version" : "3.0",
			 }
			 
		data = self.urlopen2(account, 'https://docs.google.com/feeds/default/private/full/', headers=headers)
		
		if data == None:
			return True
		
		node = ET.XML(data)
		
		self.dumpit(node)
		
		for entry in node.findall("{http://www.w3.org/2005/Atom}entry"):
			entry_id = entry.find("{http://www.w3.org/2005/Atom}id").text
			c = entry_id.find("%3A")
			entry_id = entry_id[c+3:]
			
			entry_title = entry.find("{http://www.w3.org/2005/Atom}title").text
			
			
			event_content = entry.find("{http://www.w3.org/2005/Atom}content")
			type = event_content.attrib["type"]

			upd = entry.find("{http://www.w3.org/2005/Atom}updated")
			tmpstr = upd.text[:-5] + " UTC"
			updtime = calendar.timegm(time.strptime(tmpstr, "%Y-%m-%dT%H:%M:%S %Z"))
			
			
			print entry_title + ": ",
			print updtime

			viewed = False
			for i in entry.findall("{http://www.w3.org/2005/Atom}category"):
				if i.attrib["label"] == "viewed":
					viewed = True
				else:
					print "Label " + i.attrib["label"]
				if i.attrib["scheme"].endswith("kind"):
					kind = i.attrib["label"]
					
			if not viewed and kind != "folder":
				if kind == "document":
					icon_name = "application-document.png"
				elif kind == "pdf":
					icon_name = "application-pdf.png"
				elif kind == "presentation":
					icon_name = "application-presentation.png"
				elif kind == "spreadsheet":
					icon_name = "application-spreadsheet.png"
				elif kind == "form":
					icon_name = "application-form.png"
				elif kind == "application/download":
					icon_name = "application-document.png" # fixme data
				else:
					print "??Kind is " + kind
					
				icon = os.path.join(googsystray.BASE_DIR, "icons", icon_name)
			
				new_messages.append(
					{ "id": entry_id + str(updtime) + account["name"],
					  "sub_id": "doc",
					  "account": account["name"],
					  "count": 1,
					  "title": _("Google Document Updated"),
					  "summary": entry_title,
					  "icon": icon,
					  "marked": False,
					  "data": { "kind" : kind,
					  	    "id" : entry_id,
						    "upd_time" : str(updtime),
						    "title" : entry_title,
						    "labels" : "None",
					  	 },
					})
					#fixme upd_time currently needs to be a string
		"""
		
		self.merge_messages(account["name"], new_messages)

		return True
		
				
	def __init__(self):
		self.config = "gdocs"
		self.auth = {}
		
		GIcon.GIcon.__init__(self)
