# -*- coding: utf-8 -*-

import socket
import sys
import traceback
import gobject
import gtk
import GMain

# To be honest, I'd much rather have used some sort of shared-memory IPC
# as I imagine some Windows firewall software is probably going to screw
# this up.  But I don't want to maintain two different things for Windows
# and *nix support.


class GIPC:
	def client_recv(self, source, cond):
		try:
			msg = source.recv(512)
			if len(msg) == 0:
				for i in self.clients:
					if i["socket"] == source:
						gobject.source_remove(i["gid"])
						self.clients.remove(i)
						return True
				print "Shouldn't get here"
				return True
			if msg == "CFG":
				gobject.idle_add(GMain._GConf.open_config_window, None)
			elif msg == "DBG":
				self.dump_all()
			elif msg[:5] == "MAIL:":
				print "Mail: " + msg[5:]
				# We're not part of the whole icon/base thing
				# so just use GMain's
				gmail = GMain._GMain.get_service("gmail")
				gobject.idle_add(gmail["class"].parse_mailto, msg[5:])
				
		except socket.error, e:
			pass
		return True
			
	def server_connect(self, source, cond):
		try:
			(clientsocket, address) = self.serversocket.accept()
			gid = gobject.io_add_watch(clientsocket, gobject.IO_IN, self.client_recv)
			self.clients.append({"gid":gid,"socket":clientsocket})
		except:
			pass
		return True
	

	def start_server (self):
		self.serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.serversocket.bind(('127.0.0.1', 49815))
		self.serversocket.listen(5)
		self.server_gid = gobject.io_add_watch(self.serversocket, gobject.IO_IN, self.server_connect)
		
		
	def start_client (self):
		try:
			self.clientsocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
			self.clientsocket.connect(('127.0.0.1', 49815))
		except:
			self.clientsocket = None

	def is_client(self):
		if self.clientsocket == None:
			return False
		return True

	def dump_all(self):
		code = []
		for threadId, stack in sys._current_frames().items():
			code.append("\n# ThreadID: %s" % threadId)
			for filename, lineno, name, line in traceback.extract_stack(stack):
				code.append('File: "%s", line %d, in %s' % (filename, lineno, name))

				if line:
					code.append("  %s" % (line.strip()))
		print "\n".join(code)

		error_dlg = gtk.Dialog(_("Googsystray error."), None, gtk.DIALOG_MODAL, 
			buttons = (gtk.STOCK_OK, gtk.RESPONSE_OK))
		error_dlg.set_default_response(gtk.RESPONSE_OK)
		label1 = gtk.Label()
		label1.set_markup(_("<b>Googsystray has experienced an error.  Please file a bug report at http://googsystray.sf.net, attaching the following stack trace:</b>"))
		label1.set_line_wrap(True)
		label2 = gtk.Label("\n".join(code))
		label2.set_selectable(True)
		
		error_dlg.vbox.pack_start(label1, False, False, 5)
		error_dlg.vbox.pack_start(label2, False, False, 5)
			
		error_dlg.show_all()
		error_dlg.run()
		error_dlg.destroy()
		print "Out"

	def debug_trace (self):
		self.clientsocket.send("DBG")

	def start_config (self):
		self.clientsocket.send("CFG")
		
	def send_mail (self, mailto):
		self.clientsocket.send("MAIL:"+mailto)

	def __init__ (self):
		self.clientsocket = None
		self.serversocket = None
		self.clients = []
