# -*- coding: utf-8 -*-

import googsystray
import threading
import gtk, gobject
import webbrowser
import copy
import time
import os
import sys


import GConf
import GMain
from GBase import GBase
import GPopupWindow
import GNotification

_font_pb = None


#########################################################
# Base Icon Class
#########################################################

class link_label(gtk.EventBox):

	def set_markup(self, markup):
		self.label.set_markup(markup)

	def label_enter(self, widget, data=None):
		cursor = gtk.gdk.Cursor(widget.get_display(), gtk.gdk.HAND2)
		window = widget.window
		window.set_cursor(cursor)
		if self.popup_timeout != -1:
			self.popup_timer = gobject.timeout_add(self.popup_timeout,
							       self.popup_fun,
							       self.popup_data)

	def label_leave(self, widget, data=None):
		if self.popup_timer != -1:
			gobject.source_remove(self.popup_timer)
			self.popup_timer = -1
		window = widget.window
		window.set_cursor(None)
		
	def set_popup_func(self, function, data, timeout):
		self.popup_fun = function
		self.popup_data = data
		self.popup_timeout = timeout
		
	def __init__(self, text=None, clickfun=None, clickdata=None):
		gtk.EventBox.__init__(self)

		self.popup_fun = None
		self.popup_data = None
		self.popup_timeout = -1
		self.popup_timer = -1
		self.label = gtk.Label()
		if text is not None:
			self.label.set_markup(text)
		self.add(self.label)	
		self.set_events(gtk.gdk.BUTTON_PRESS_MASK |
				  gtk.gdk.ENTER_NOTIFY_MASK |
				  gtk.gdk.LEAVE_NOTIFY_MASK)
		self.set_visible_window(False)
		self.connect("enter_notify_event", self.label_enter)
		self.connect("leave_notify_event", self.label_leave)
		if clickfun:
			self.connect("button_press_event", clickfun, clickdata)


class GIcon(GBase):

	def new_image_labelsize(self, fname):
		try:
			pb = gtk.gdk.pixbuf_new_from_file(os.path.join(googsystray.BASE_DIR, "icons", fname))
		except:
			# This shouldn't happen... don't send crappy icons.
			return None

		#FIXME you should probably be caching these for performance
		#as they'll be used many times

		h = int(16 * self.get_gconfig()["popup_size"])			
			
		pb2 = pb.scale_simple(h, h, gtk.gdk.INTERP_HYPER)
		image = gtk.image_new_from_pixbuf(pb2)
		return image

	def label_enter(self, widget, data=None):
		cursor = gtk.gdk.Cursor(widget.get_display(), gtk.gdk.HAND2)
		window = widget.window
		window.set_cursor(cursor)

	def label_leave(self, widget, data=None):
		window = widget.window
		window.set_cursor(None)
		
	def new_image_button(self, im, clickfun, data=None):
		image = self.new_image_labelsize(im)
		ebox = gtk.EventBox()
		ebox.add(image)
		ebox.set_events(gtk.gdk.BUTTON_PRESS_MASK |
				  gtk.gdk.ENTER_NOTIFY_MASK |
				  gtk.gdk.LEAVE_NOTIFY_MASK)
		ebox.set_visible_window(False)
		ebox.connect("button_press_event", clickfun, data)
		ebox.connect("enter_notify_event", self.label_enter)
		ebox.connect("leave_notify_event", self.label_leave)
		return ebox
		

	def _send_post_mark(self, data):
		account = data["account"]
		url = data["url"]
		message = data["message"]
		params = data["params"]
		data = self.urlopen2(account, url, params)
		
		if data == None:
			return
			
		self.thread_lock.acquire()
		for i in self.messages_unread:
			if i["id"] == message["id"]:
				i["marked"] = True
				break
		self.thread_lock.release()

		# Check again, so the unread/read status properly
		# gets into the system		
		if self.thread_cond.acquire(0):
			self.thread_cond.notify()
			self.thread_cond.release()

		gobject.idle_add(self.update_icon, None)
		return
		

	def send_post_mark(self, account, url, message, params=None):

		args=[{"url":url,
		       "account":account,
		       "message":copy.deepcopy(message),
		       "params":params,
		       }]
		threading.Thread(target=self._send_post_mark, args=args).start()



	def set_sensitives(self):
		for i in self.sens_widgets:
			i.set_sensitive(False)

		if not self.am_used(True):
			return
		
		for i in self.sens_widgets:
			if i.get_data("make_sensitive") != None:
				i.set_sensitive(True)
			if i.get_data("sf_label") != None:
				i.set_sensitive(True)
				if i.get_active():
					i.get_data("sf_label").set_sensitive(True)
					i.get_data("sf_button").set_sensitive(True)
			if i.get_data("cmd_label") != None:
				i.set_sensitive(True)
				if i.get_active():
					i.get_data("cmd_label").set_sensitive(True)
					i.get_data("cmd_entry").set_sensitive(True)
			if i.get_data("onclick_entry") != None:
				i.set_sensitive(True)
				if i.get_active():
					i.get_data("onclick_entry").set_sensitive(True)


		
	def check_toggled(self, widget, data=None):
		self.get_backupconfig()[data] = widget.get_active()
		self.set_sensitives()
		
	def spinner_changed(self, adj, conf):
		self.get_backupconfig()[conf] = int(adj.get_value())
		
	def slider_moved(self, adj, conf):
		self.get_backupconfig()[conf] = int(adj.get_value()*60000.0)
	

	def sound_button_callback(self, widget, data=None):
		data["enable_sound"] = widget.get_active()
		self.set_sensitives()

	def command_button_callback(self, widget, data=None):
		data["enable_other"] = widget.get_active()
		self.set_sensitives()
			
	def play_clicked(self, widget, data=None):
		if (data.get_filename() != None):
			play_sound(data.get_filename())

			
	def hotkey_release(self, widget, event, data=None):
		if not data.get_data("capturing"):
			return False
		keyname = gtk.gdk.keyval_name(event.keyval)
		if keyname == "Meta_L" or keyname == "Meta_R":
			self.meta_status = False

			
	def hotkey_press(self, widget, event, data=None):
		if not data.get_data("capturing"):
			return False
		str = ""
		keyname = gtk.gdk.keyval_name(event.keyval)

		if os.name == "nt":
			if keyname == "Meta_L" or keyname == "Meta_R":
				self.meta_status = True
		if event.state & gtk.gdk.CONTROL_MASK:
			str += "<Control>"
		if event.state & gtk.gdk.SHIFT_MASK:
			str += "<Shift>"
		if event.state & gtk.gdk.SUPER_MASK or self.meta_status:
			str += "<Super>"
		if event.state & gtk.gdk.MOD1_MASK:
			str += "<Alt>"
		str += keyname.upper()
		data.set_label(str)
		return True


	def hotkey_clicked(self, widget, data=None):
		self.meta_status = False
		dialog = gtk.Dialog(_("Select New Hotkey"), None, gtk.DIALOG_MODAL,
					(
					 _("Capture"), 100,
					 _("Clear/Disable"), 101,
					 gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
					 gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,
					 ))
		hbox = gtk.HBox()
		label = gtk.Label(_("Hotkey Combination:"))
		label2 = gtk.Label(data["keys"])
		label2.set_data("capturing", False)
		hbox.pack_start(label, False, False, 2)
		hbox.pack_end(label2, False, False, 2)
		dialog.vbox.pack_start(hbox, False, False, 2)
		dialog.connect("key-press-event", self.hotkey_press, label2)
		if os.name == "nt":
			dialog.connect("key-release-event", self.hotkey_release, label2)
		dialog.add_events(gtk.gdk.KEY_PRESS_MASK | gtk.gdk.KEY_RELEASE_MASK)
		hbox.show_all()

		while True:
			ret = dialog.run()
			if ret == 100: # Capture
				label2.set_data("capturing", True)
				label2.set_label("")
			elif ret == 101: # Clear
				label2.set_label("<None>")
			elif ret == gtk.RESPONSE_ACCEPT:
				data["keys"] = label2.get_label()
				widget.set_label(data["keys"])
				dialog.destroy()
				break
			elif (ret == gtk.RESPONSE_REJECT or 
			      ret == gtk.RESPONSE_DELETE_EVENT):
			      	dialog.destroy()
			      	break
			
		return

	def edit_notify(self, widget, tv):
		(model, rows) = tv.get_selection().get_selected_rows()

		if len(rows) == 0:
			return True
			
		acct = None
		notify = None
			
		for a in self.get_gbackupconfig()["accounts"].items():
			if a[0] == model[rows[0]][0]:
				acct = a[1]
				break
		
		for n in self.get_backupconfig()["sub_notifications"]:
			if n["name"] == model[rows[0]][1] and n["account"] == a[0]:
				notify = n
				break
				
		if acct is None or notify is None:
			print "ERROR shouldn't be here fixme"
			return

		not_dlg = gtk.Dialog(_("Edit Notification"), None, gtk.DIALOG_MODAL, (
					 gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
					 gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,
					 ))


		vbox = not_dlg.vbox
		
		hbox1 = gtk.HBox()
		vbox.pack_start(hbox1, False, False, 2)
		snd_cbutton = gtk.CheckButton(_("Play sound"))
		snd_cbutton.set_tooltip_text(_("Play a sound on a new item."))
		snd_cbutton.connect("toggled", self.sound_button_callback, notify)
		snd_cbutton.set_active(notify["enable_sound"])
		hbox1.pack_start(snd_cbutton, False, False, 2)

		hbox = gtk.HBox()
		label = gtk.Label(_("Sound File:"))
		label.set_tooltip_text(_("What sound file to play when a new item is found."))
		button = gtk.FileChooserButton("Sound File")
		button.set_filename(GNotification.get_sound_file(notify["sound_file"]))
			
		preview_button = gtk.Button(stock=gtk.STOCK_MEDIA_PLAY)
		button.set_extra_widget(preview_button)
		preview_button.connect("clicked", self.play_clicked, button)
		hbox.pack_start(label, False, False, 2)
		hbox.pack_start(button, True, True, 2)
		vbox.pack_start(hbox, False, False, 2)

		run_cbutton = gtk.CheckButton(_("Run Command"))
		run_cbutton.set_tooltip_text(_("Run a command on a new item."))
		run_cbutton.connect("toggled", self.command_button_callback, notify)
		run_cbutton.set_active(notify["enable_other"])
		hbox1.pack_start(run_cbutton, False, False, 2)
			
		hbox = gtk.HBox()
		label = gtk.Label(_("Run Command:"))
		label.set_tooltip_text(_("What command to run when a new item is found."))
		entry = gtk.Entry()
		if notify["other_command"] != None:
			entry.set_text(notify["other_command"])
		hbox.pack_start(label, False, False, 2)
		hbox.pack_start(entry, True, True, 2)
		vbox.pack_start(hbox, False, False, 2)

		

		not_dlg.show_all()
		rsp = not_dlg.run()
		
		if rsp == gtk.RESPONSE_REJECT:
			not_dlg.destroy()
			return
			
		notify["enable_sound"] = model[rows[0]][3] = snd_cbutton.get_active()
		notify["enable_other"] = model[rows[0]][4] = run_cbutton.get_active()
		snd_file = notify["sound_file"]
		if button.get_filename() != GNotification.get_sound_file(snd_file):
			if os.path.dirname(button.get_filename()) == os.path.join(googsystray.BASE_DIR, "sounds"):
				notify["sound_file"] = "$" + os.path.basename(button.get_filename())
			else:
				notify["sound_file"] = button.get_filename()
		notify["other_command"] = entry.get_text()
			
			
		not_dlg.destroy()
				
			
	def update_notification_buttons(self):
		store = self.treeview.get_model()
		is_treeview = self.get_backupconfig()["not_treeview"]
		store.clear()
		
		for a in self.get_gbackupconfig()["accounts"].items():
			parent = None
			for n in self.get_backupconfig()["sub_notifications"]:
				if n["account"] != a[0]:
					continue
					
				if not a[1]["services"][self.config]:
					continue
					
				data = [a[0], 
					n["name"], 
					not n["hidden"], 
					n["enable_sound"], 
					n["enable_other"], 
					n["sound_file"], 
					n["other_command"]]
				if is_treeview:
					iter = store.append(parent, data)
					if parent is None:
						parent = iter
				else:
					store.append(data)
	

	def toggle_notification(self, cell, path, data=None):
		tv = data
		model = tv.get_model()
		
		if model[path][1] == "Inbox email": # Can't turn off Inbox
			return
			
		for i in self.get_backupconfig()["sub_notifications"]:
			if i["name"] == model[path][1] and i["account"] == model[path][0]:
				i["hidden"] = model[path][2]
				model[path][2] = not model[path][2]

			

	def draw_notification_buttons(self, vbox):
		if "sub_notifications" not in self.get_backupconfig():
			# not a 'standard' sound interface, so it'll be covered elsewhere
			return
			
		
		is_treeview = self.get_backupconfig()["not_treeview"]
		allow_hide = self.get_backupconfig()["not_allowhide"]
				
		if is_treeview:
			store = gtk.TreeStore(str,str,bool,bool,bool,str,str)
		else:
			store = gtk.ListStore(str,str,bool,bool,bool,str,str)
		
		tv = gtk.TreeView(store)
		tv.get_selection().set_mode(gtk.SELECTION_SINGLE)

		cell = gtk.CellRendererText()
		col = gtk.TreeViewColumn("Account", cell)
		col.add_attribute(cell, 'text', 0)
		tv.append_column(col)

		cell = gtk.CellRendererText()
		col = gtk.TreeViewColumn("On: ", cell)
		col.add_attribute(cell, 'text', 1)
		col.set_expand(True)
		tv.append_column(col)

		

		
		if allow_hide:
			cell = gtk.CellRendererToggle()
			cell.activatable = True
			cell.connect("toggled", self.toggle_notification, tv)
			col =  gtk.TreeViewColumn(_("Enabled"), cell)
			col.add_attribute(cell, 'active', 2)
			tv.append_column(col)


		cell = gtk.CellRendererToggle()
		cell.activatable = False
		col =  gtk.TreeViewColumn(_("Play Sound"), cell)
		col.add_attribute(cell, 'active', 3)
		tv.append_column(col)

		cell = gtk.CellRendererToggle()
		cell.activatable = False
		col =  gtk.TreeViewColumn(_("Run Command"), cell)
		col.add_attribute(cell, 'active', 4)
		tv.append_column(col)



		sw = gtk.ScrolledWindow()
		sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
		# FIXME hardcoded.  I think it's OK though. 
		if self.config == "gmail":
			sw.set_size_request(-1, 200)	
		else:
			sw.set_size_request(-1, 100)
		sw.add(tv)
		
		hbox = gtk.HBox()
		hbox.pack_start(sw, True, True, 5)
		vbox2 = gtk.VBox()
		hbox.pack_start(vbox2, False, False, 5)
		button = gtk.Button(None, gtk.STOCK_EDIT)
		button.connect("clicked", self.edit_notify, tv)
		vbox2.pack_start(button, False, False, 2)
		
		vbox.pack_start(hbox, False, False, 0)
		
		
		self.treeview = tv
		self.update_notification_buttons()
		
		
	def draw_hotkey_settings(self, vbox):
		if len(self.get_backupconfig()["hotkeys"]) == 0:
			return
			
		frame = gtk.Frame(_("Global Hotkeys"))
		frame.set_tooltip_text(_("Global hotkeys will be active even when no window is open."))
		vbox.pack_start(frame, False, False, 2)
		vbox1 = gtk.VBox()
		frame.add(vbox1)
			
		for i in self.get_backupconfig()["hotkeys"].items():
			hbox = gtk.HBox()
			label = gtk.Label(_(i[1]["name"]))
			hbox.pack_start(label, False, False, 2)
			button = gtk.Button(i[1]["keys"])
			button.set_size_request(200, -1)
			button.connect("clicked", self.hotkey_clicked, i[1])

			label.set_data("make_sensitive", 1)
			self.sens_widgets.append(label)
			button.set_data("make_sensitive", 1)
			self.sens_widgets.append(button)

			hbox.pack_end(button, False, False, 2)
			vbox1.pack_start(hbox, False, False, 2)


	def close_config(self, saved):
		self.config_vbox = None
		return

	def draw_config(self, vbox):
		self.sens_widgets = []
		self.entries = []
		self.sound_buttons = []
		self.config_vbox = vbox

		hbox = gtk.HBox()
		
		button = gtk.CheckButton(_("Always show icon"))
		button.set_tooltip_text(_("Always display the icon for this service in the system tray, even if no notifications are pending."))
		button.connect("toggled", self.check_toggled, "always_show_icon")
		button.set_active(self.get_backupconfig()["always_show_icon"])
		button.set_data("make_sensitive", 1)
		self.sens_widgets.append(button)
		hbox.pack_start(button, False, True, 2)
		
		if self.config != "global":
			button = gtk.CheckButton(_("Dim icon"))
			button.set_tooltip_text(_("Dim the icon if no messages are currently waiting."))
			button.connect("toggled", self.check_toggled, "dim_icon")
			button.set_active(self.get_backupconfig()["dim_icon"])
			button.set_data("make_sensitive", 1)
			self.sens_widgets.append(button)
			hbox.pack_start(button, False, True, 20)
		
			button = gtk.CheckButton(_("Display Counts"))
			button.set_tooltip_text(_("Overlay the count of current new messages onto the systray icon."))
			button.connect("toggled", self.check_toggled, "display_count")
			button.set_active(self.get_backupconfig()["display_count"])
			button.set_data("make_sensitive", 1)
			self.sens_widgets.append(button)
			hbox.pack_start(button, False, True, 20)
		
		
		vbox.pack_start(hbox, False, False, 5)
		
		if self.config != "global":
			hbox = gtk.HBox()
			label = gtk.Label(_("Check frequency (minutes):"))
			label.set_tooltip_text(_("How often, in minutes, googsystray should check this service for new items."))
			hbox.pack_start(label, False, False, 5)
			timeout_adj = gtk.Adjustment(
						self.get_backupconfig()["timeout"] / 60000.0, 
								0.25, 15.0, 0.05, 0.05, 0.05)
			timeout_adj.connect("value_changed", self.slider_moved, "timeout")
			hslid = gtk.HScale(timeout_adj)
			hbox.pack_start(hslid, True, True, 15)
			label.set_data("make_sensitive", 1)
			hslid.set_data("make_sensitive", 1)
			self.sens_widgets.append(label)
			self.sens_widgets.append(button)
			vbox.pack_start(hbox, False, False, 2)

		
		
		self.draw_hotkey_settings(vbox)
		
		self.draw_notification_buttons(vbox)
		
		self.set_sensitives()

		
	def update_popup(self):
		self.thread_lock.acquire()
		add_msgs = []
		did_hide = False
		num_vis = 0
		
		# Remove messages that have been marked or that have otherwise
		# disappeared
		for i in self.popup_messages:
			id = i.get_data("msg_id")
			found = False
			for j in self.messages_unread:
				if j["id"] == id:
					found = True
					if j["marked"] and i.props.visible:
						did_hide = True
						i.hide()
			if not found:
				if i.props.visible:
					did_hide = True
					i.hide()
			if i.props.visible:
				num_vis += 1
			
		# Add new messages
		for i in self.messages_unread:
			found = False
			for j in self.popup_messages:
				if j.get_data("msg_id") == i["id"]:
					found = True
					break
			if not found:
				num_vis += 1
				self.draw_message(i)
				
		if num_vis == 0:
			self.popup_window.destroy()
		elif did_hide:
			self.popup_window.window.reshow_with_initial_size()
		
		self.thread_lock.release()
	
	def draw_popup(self):
		self.thread_lock.acquire()
		
		self.popup_messages = []
		for i in self.messages_unread:		
			if i["marked"]:
				continue
			self.draw_message(i)
				
		self.thread_lock.release()


		
	def can_popup(self):
		self.thread_lock.acquire()
		for i in self.messages_unread:
			if not i["marked"]:
				self.thread_lock.release()
				return True

		self.thread_lock.release()
		return False
	
	def destroy_popup(self):
		self.popup_messages = None
		self.popup_window = None
			
	def activate( self, widget, data=None):
		if self.popup_window is not None:
			self.popup_window.destroy()
			self.popup_window = None
			return
			
		if not self.can_popup():
			return

		(m_x, m_y, m_mask) = self.staticon.get_screen().get_root_window().get_pointer()

		print m_x
		print m_y
		print m_mask

		self.popup_window = GPopupWindow.get_new_popup_window(self.destroy_popup, 
								      self.get_config()["service_name"], 
								      m_x, m_y,
								      icon=os.path.join(googsystray.BASE_DIR, "icons", self.get_config()["icon"]))
								      
		self.draw_popup()
		self.popup_window.show_all()

		return
		
	def set_icon_numeric(self, num):
		num_pb = self.staticon_pb.copy()
		global _font_pb
		if _font_pb is None:
			_font_pb = gtk.gdk.pixbuf_new_from_file(os.path.join(googsystray.BASE_DIR, "icons", "Num_Font.png"))
		num_str = str(num)

		x = 0
		for i in num_str:
			_font_pb.copy_area(int(i)*22,0,21,32,num_pb,x,0)
			x += 21
			
		self.staticon.set_from_pixbuf(num_pb)
		

	def make_tooltip(self):
		cnt = 0
		for i in self.messages_unread:
			if not i["marked"]:
				cnt += i["count"]
		return ngettext("%(count)d new message.", "%(count)d new messages.", cnt) % {"count":cnt}

	def update_icon(self, data=None):
		for i in GMain._services:
			i["class"].update_icon_full()
	
	
	def update_icon_full(self, data=None):
		if not self.am_used():
			return
			
		# This shouldn't happen, but we might not be initialized yet.
		if self.staticon is None:
			return
			
		if self.popup_window is not None:
			self.update_popup()
		
		self.thread_lock.acquire()
		
		self.staticon.set_tooltip(self.make_tooltip())
		
		min_unread = 0
		if self.get_config().get("minimum_unread"):
			min_unread = self.get_config()["minimum_unread"]

		cnt = 0
		old_cnt = 0
		for i in self.messages_unread_old:
			if not i["marked"]:
				old_cnt += i["count"]
		for i in self.messages_unread:
			if not i["marked"]:
				cnt += i["count"]

#		print "Cnt: " + str(cnt) + "  Old_cnt:" + str(old_cnt)

		if (self.messages_unread == None or cnt == 0 or cnt < min_unread):
			if not self.get_config()["always_show_icon"]:
				self.staticon.set_visible(False)
				self.thread_lock.release()
				return
			if self.get_config()["dim_icon"]:
				self.staticon.set_from_pixbuf(self.staticon_pb_grey)

			if old_cnt != 0 and self.get_config()["display_count"]:
				if cnt == 0:
					self.staticon.set_from_pixbuf(self.staticon_pb)
				else:
					self.set_icon_numeric(cnt)

			self.messages_unread_old = []
			self.thread_lock.release()
			return
			
		cats = []

		# icon is _always_ visible with waiting data
		if not self.staticon.get_visible():
			self.staticon.set_visible(True)

		for i in self.messages_unread:
			found = False
			for j in self.messages_unread_old:
				if (i["account"] != j["account"] or
				    i["id"] != j["id"] or
				    i["sub_id"] != j["sub_id"]):

					continue

				found = True
				break
			
			if found: # We have already notified on this item.
				continue
			
			for j in cats:
				if j == (i["account"] + i["sub_id"]):
					found = True
					break
					
			if found: # We have already notified on this category
				  # this round
				GNotification.notify(self, i, False)
				continue
		
			if min_unread == 0:
				GNotification.notify(self, i, True)
				
			cats.append((i["account"] + i["sub_id"]))
			

		# This next block of code is pretty specifically for reader.
		if min_unread != 0:
			if cnt >= min_unread:
				if cnt > old_cnt:
					if self.notify_next:
						GNotification.notify(self, self.messages_unread[0], True)
				if self.get_config()["minimum_notify_once"]:
					self.notify_next = False
			else:
				self.notify_next = True
				

		if (cnt != old_cnt and self.get_config()["display_count"]):
			self.set_icon_numeric(cnt)
		elif len(self.messages_unread_old) == 0 and self.get_config()["dim_icon"]:
			self.staticon.set_from_pixbuf(self.staticon_pb)

			
		self.messages_unread_old = copy.deepcopy(self.messages_unread)
			
		self.thread_lock.release()
			
		return False
			

	def merge_messages(self, account_name, new_messages):
		# First, remove any messages from this account that are not
		# in the new_messages
		done = False
		
		for i in new_messages:
			i["type"] = self.config
		
		while not self.thread_lock.acquire(0):
			if not self.thread_run:
				return
			time.sleep(0.1)
			
		
		while not done:
			done = True
			cnt = 0
			for i in self.messages_unread:
				if i["account"] != account_name:
					continue
				found = False
				for j in new_messages:
					if i["id"] == j["id"]:
						found = True
						break
				if not found:
					self.messages_unread.remove(i)
					done = False
					break
				cnt += 1
		
		# Next, we add all new_messages to the list that aren't
		# already in there.  This way we keep the 'marked'
		# status.
		
		for i in new_messages:
			found = False
			for j in self.messages_unread:
				if i["id"] == j["id"] and j["account"] == account_name:
					j["data"] = copy.deepcopy(i["data"])
					found = True
					break
			if not found:
				self.messages_unread.append(i)
		self.thread_lock.release()
		gobject.idle_add(self.update_icon, None)
		

	def icon_stop(self):
		return
		
	def stop (self):
		if self.staticon is not None:
			self.staticon.set_visible(False)
		self.stop_timer()
		self.icon_stop()
		return
		
	def icon_start (self):
		return
		
	def start (self):
		if not self.am_used():
			return
			
		self.messages_unread = []
		self.messages_unread_old = []
		self.notify_next = True
		
		if self.menu is not None:
			self.menu.destroy()
		self.menu = gtk.Menu()
		self.create_menu()
			
		if self.staticon == None:
			self.staticon = gtk.StatusIcon()
			self.staticon_pb = gtk.gdk.pixbuf_new_from_file(os.path.join(googsystray.BASE_DIR, "icons", self.get_config()["icon"]))
			self.make_pb_grey()

			self.staticon.connect("activate", self.activate)
			self.staticon.connect("popup_menu", self.popup)

			if self.get_config()["dim_icon"]:
				self.staticon.set_from_pixbuf(self.staticon_pb_grey)
			else:
				self.staticon.set_from_pixbuf(self.staticon_pb)

		if self.get_config()["always_show_icon"]:
			self.staticon.set_visible(True)
		else:
			if self.staticon.get_visible():
				self.staticon.set_visible(False)
				
		self.icon_start()

		self.timeout = self.get_config()["timeout"]
		self.start_timer()

	def make_pb_grey(self):
		# What a fucking nightmare!  Screw you, pygtk win32 distro
		# for not having Numeric support and supporting 
		# get_pixels_array()!  Also, screw you Python for having
		# immutable strings.
		data = ""
		rs = self.staticon_pb.get_rowstride()
		if self.staticon_pb.get_has_alpha():
			has_alpha = True
			ps = 4
		else:
			ps = 3
			has_alpha = False
		
		pix = self.staticon_pb.get_pixels()
		nr = self.staticon_pb.get_height()
		w = self.staticon_pb.get_width()
		row = 0

		while row < nr:
			i = 0
			while i < w:
				v = int(float(ord(pix[row*rs+i*ps+0])) * 0.30 +
				     float(ord(pix[row*rs+i*ps+1])) * 0.59 +
				     float(ord(pix[row*rs+i*ps+2])) * 0.11)
				
				data += chr(v) + chr(v) + chr(v)
				
				if has_alpha:
					data += pix[row*rs+i*ps+3]

				
				i += 1	
			
			# Pad the row.  I don't think this will get hit
			# on any of our png's, though, being 64 pixels in 
			# width and having an alpha channel.
			c = 0
			while (i*ps+c<rs):
				data += chr(0)
			row += 1
		
		
		self.staticon_pb_grey = gtk.gdk.pixbuf_new_from_data(data,
								     gtk.gdk.COLORSPACE_RGB,
								     has_alpha,
								     8,
								     w,
								     nr,
								     rs)
								     
		

	# anything that won't display properly in a label
	def fix_markup(self, data):
		start = 0

		c = data.find('&')
		while (c != -1):
			start = c
			
			ce = data.find(';', c, c+7)
			if ce == -1:
				data = data[:c] + "&amp;" + data[c+1:]
			else:
				start = start + 1

			c = data.find('&', start)
	
		return data
	
	def xmlize_html(self, data):
		data = data.replace("&nbsp;", " ")
		data = data.replace("&lrm;", "")
		data = data.replace("&rlm;", "") # I suppose we should handle
						 # this in some other way
		close_list = ["br", "img", "hr", "input", "wbr", "col", "wbr"]
		ldata = data.lower()
		for i in close_list:
			c = 0
			while True:
				c = ldata.find("<" + i, c)
				if c == -1:
					break
					
				c1 = ldata.find(">", c)
				if c1 == -1:
					print "Malformed HTML!!!!"
					
				if ldata[c1-1] == "/":
					c = c1
					continue
						
				data = data[:c1] + "/" + data[c1:]
				ldata = data.lower()
				c = c1

		fix_list = ["td"]
		ldata = data.lower()
		for i in fix_list:
			c = 0
			while True:
				c = ldata.find("<" + i, c)
				if c == -1:
					break
					
				c1 = ldata.find(">", c)
				if c1 == -1:
					print "Malformed HTML!!!!"
					
				# Ugh.  nowrap="nowrap" is proper, but it appears
				# some just use 'nowrap'. I think it's just <td> tags
				
				if i == "td":
					c2 = ldata.find("nowrap", c, c1)
					if c2 != -1:
						if ldata[c2+6] != "=":
							data = data[:c2+6] + "=\"nowrap\"" + data[c2+6:]
							ldata = data.lower() 
						
				c = c1

		return data


	def strip_html(self, data, fix_amps=True):
		# There is more than likely a better way to cut out
		# &nbsp; etc but I'm too lazy to care.
		# Uh oh!  It looks like on reader feeds, google can be really messed up
		# and send us stuff like '&amp;#39;'.  Cute.  So we've got to do
		# an initial pass just for '&amp;' and turn it into '&'.  Except
		# sometimes, where we just need to turn loose ampersands into
		# &amps;.  Grr. 
		
		start = 0

		c = data.find('&')
		while (c != -1):
			start = c
			
			ce = data.find(';', c, c+7)
			if ce == -1 and fix_amps:
				data = data[:c] + "&amp;" + data[c+1:]
			elif data[c:ce+2] == "&amp;#":
				# special case
				data = data[:c] + "&" + data[ce+1:]
			elif data[c:ce] == "&nbsp":
				data = data[:c] + " " + data[ce+1:]
#			elif data[c:ce] == "&#39":
#				data = data[:c] + "'" + data[ce+1:]
			elif data[c:ce] == "&raquo":
				# we really don't care about this
				data = data[:c] + " " + data[ce+1:]
			elif data[c:ce] == "&copy":
				# we really don't care about this
				data = data[:c] + " " + data[ce+1:]
			else:
				start = start + 1

			c = data.find('&', start)
		return data

	def get_config(self):
		return GMain._GConf.gconfig[self.config]
		
	def get_backupconfig(self):
		return GMain._GConf.backupconfig[self.config]

	def num_used(self, backup=False):
		if backup:
			conf = GMain._GConf.backupconfig
		else:
			conf = GMain._GConf.gconfig
			
		c = 0
			
		for i in conf["global"]["accounts"].items():
			if i[1]["services"][self.config]:
				c += 1
		return c

	def am_used(self, backup=False):
		if self.config == "global":
			return True
		
		if self.num_used(backup) > 0:
			return True
			
		return False

	def unbind_hotkey(self, name):
		(mod, key) = GConf.parseHotKey(self.get_config()["hotkeys"][name]["keys"])
		if key != None:
			GMain._GMain.unbindKey(key, mod)
		
	def bind_hotkey(self, name, func, data=None):
		(mod, key) = GConf.parseHotKey(self.get_config()["hotkeys"][name]["keys"])
		if key != None:
			GMain._GMain.bindKey(key, mod, func, data)


	def icon_check_data(self, account):
		return

	def check_data(self):
		for i in GMain._accounts:
			if i["services"][self.config]:
				self.icon_check_data(i)
	

	def start_timer(self):
		self.timeout = self.get_config()["timeout"]
		GBase.start_timer(self)


	def get_service(self, name):
		for i in GMain._services:
			if name == i["service"]:
				return i
		return None

	def quit_cb(self, widget, data=None):
		GMain.stop_services()
#		sys.exit(1)
		gtk.main_quit()
		
	def config_cb(self, widget, data=None):
		GMain._GConf.open_config_window()
		return
		
		
	def about_close(self, dialog, resp_id, data=None):
		dialog.destroy()

	def emailfun(self, dialog, link, data=None):
		webbrowser.open("mailto:"+link)
		return
	
	def urlfun(self, dialog, link, data=None):
		print link
		# this never even gets called... wtf?
		return

	def about_cb(self, widget, data=None):
		#gtk.about_dialog_set_url_hook(lambda dialog, url: webbrowser.open(url))
		gtk.about_dialog_set_email_hook(lambda dialog, url: webbrowser.open("mailto:" +url))
		about = gtk.AboutDialog()
		about.set_name("googsystray")
		about.set_program_name("googsystray")
		about.set_version(googsystray.VERSION_STRING)
		about.set_copyright("Copyright 2009 Jim Duchek <jim.duchek@gmail.com>")
		about.set_authors(["Jim Duchek <jim.duchek@gmail.com>"])
		
		# This is a special string -- translators, put your name + email address here
		about.set_translator_credits(_("translator-credits"))
		about.set_website("http://googsystray.sourceforge.net")
		about.set_website_label("http://googsystray.sourceforge.net")

		about.connect("response", self.about_close, None)		
		about.set_keep_above(True)
		about.show_all()

	def goto_url(self, widget, url=None):
		self.browser_open(url)

	def add_menu_items(self, menu):
		service = self.get_service(self.config)
		main_service = False
		acct_services = {}
		for i in GMain._accounts:
			if i["services"][self.config]:
				if i["domain"] is None and not main_service:
					menuItem = gtk.MenuItem(_("Go to %(url)s") % {"url":self.longname})
					menuItem.connect("activate", self.goto_url, service["url"])
					menu.append(menuItem)
					main_service = True
				if i["domain"] is not None and not i["domain"] in acct_services:
					menuItem = gtk.MenuItem(_("Go to %(url)s (%(domain)s)") % {"url":self.longname,"domain":i["domain"]})
					menuItem.connect("activate", self.goto_url, service["apps_url"] % {"domain":i["domain"]})
					menu.append(menuItem)
					acct_services[i["domain"]] = True
					
					
				
				

#		if self.url is not None:
#			menuItem = gtk.MenuItem(_("Go to %(url)s") % {"url":self.longname})
#			menuItem.connect("activate", self.goto_url, self.url)
#			menu.append(menuItem)

	def create_menu(self):

		self.add_menu_items(self.menu)			

		self.menu.append(gtk.SeparatorMenuItem())

		menuItem = gtk.ImageMenuItem(gtk.STOCK_PREFERENCES)
		menuItem.connect("activate", self.config_cb, None)
		self.menu.append(menuItem)
		self.menu.append(gtk.SeparatorMenuItem())

		menuItem = gtk.ImageMenuItem(gtk.STOCK_ABOUT)
		menuItem.connect("activate", self.about_cb, None)
		self.menu.append(menuItem)
		menuItem = gtk.ImageMenuItem(gtk.STOCK_QUIT)
		menuItem.connect("activate", self.quit_cb, None)
		self.menu.append(menuItem)
		
		self.menu.connect("motion-notify-event", self.test_motion)

	def test_motion(self, data1=None, data2=None, data3=None):
		self.menu_touched = time.time()
		
	def menu_timeout(self, data=None):
		if not self.menu.props.visible:
			return False
			
		if (self.menu_touched + 1) < time.time():
			self.menu.popdown()
			return False
			
		return True
		
	def popup( self, widget, button, t, data=None):
		if self.menu.props.visible:
			self.menu.popdown()
		else:
			self.menu.show_all()
			self.menu.popup(None, None, None, button, t)
			self.menu_touched = time.time()
			self.menu_timer = gobject.timeout_add(1000, self.menu_timeout)
		return
		
	
	def __init__ (self):
		self.popup_window = None
		self.config_vbox = None
		self.staticon = None
		self.messages_unread = []
		self.messages_unread_old = []
		self.notify_next = True
		self.menu = None
		self.menu_timer = -1;
		self.menu_touched = -1
	
		GBase.__init__(self)
		



