# -*- coding: utf-8 -*-

import os
import GMain
from ctypes import *

try:
	import gnomekeyring as gkey
	_has_gkeyring = True
except:
	_has_gkeyring = False
	
try:
	import dbus
	_has_dbus = True
	_bus = dbus.SessionBus()
except:
	_has_dbus = False
	
class GKeyring:
	def get_creds(self, service, config):
		return (None, None)
		
	def set_creds(self, service, config, username, password):
		return
		
	def __init__ (self):
		return
		

class GKeyringGnome (GKeyring):
	def get_creds(self, service, config):
		attrs = {}
		attrs["server"] = service.encode("utf-8")

		try:
			print attrs
			items = gkey.find_items_sync(gkey.ITEM_NETWORK_PASSWORD, attrs)
			return (items[0].attributes["user"], items[0].secret)
		except gkey.NoMatchError, e:
			print "NO MATCH"
		except gkey.DeniedError, e:
			print "ERROR FETCHING PASSWORDS"
		return (None, None)
		
	def set_creds(self, service, config, username, password):
		attrs = { "server": service,
		          "user": username,}
		gkey.item_create_sync(self._keyring, gkey.ITEM_NETWORK_PASSWORD, "googsystray", attrs, password, True)
		return
		
	def __init__ (self):
		GKeyring.__init__(self)
		self._keyring = gkey.get_default_keyring_sync()
		
class GKeyringKDE (GKeyring):
	def get_creds(self, service, config):
		if self.hasFolder(self.handle, self.appid, self.appid) == 0:
			return (None, None)
		
		password = self.readPassword(self.handle, self.appid, service + "_pw", self.appid)
		username = self.readPassword(self.handle, self.appid, service + "_un", self.appid)
				
		return (username, password)
		
	def set_creds(self, service, config, username, password):
		if self.hasFolder(self.handle, self.appid, self.appid) == 0:
			self.createFolder(self.handle, self.appid, self.appid)
		
		
		self.writePassword(self.handle, self.appid, service + "_pw", password, self.appid)
		self.writePassword(self.handle, self.appid, service + "_un", username, self.appid)
		
		return
	
	def __init__ (self):
		GKeyring.__init__(self)
		# 1234?  So cheating.  Fixme, do this right.  Does it matter?
		self.id = 1234
		self.appid = "Googsystray"
		self.proxy = _bus.get_object("org.kde.kwalletd", "/modules/kwalletd")
		self.open = self.proxy.get_dbus_method("open", "org.kde.KWallet")
		self.hasFolder = self.proxy.get_dbus_method("hasFolder", "org.kde.KWallet")
		self.createFolder = self.proxy.get_dbus_method("createFolder", "org.kde.KWallet")
		self.writePassword = self.proxy.get_dbus_method("writePassword", "org.kde.KWallet")
		self.readPassword = self.proxy.get_dbus_method("readPassword", "org.kde.KWallet")
		
		self.handle = self.open("kdewallet", self.id, self.appid)
		print self.handle
		# Catch failure case!!!!!!! fixme
		

class BLOB(Structure):
	_fields_ = [("cbData", c_ulong),
	            ("pbData", c_void_p)]


class GKeyringW32 (GKeyring):
	def get_creds(self, service, config):
		if "bpassword" not in config[service]:
			return (None, None)

		bin_pw = eval(config[service]["bpassword"])
		blob = BLOB()
		blob2 = BLOB()
		blob.cbData = len(bin_pw)
		pass_str = c_char_p(bin_pw)
		blob.pbData = cast(pass_str, c_void_p)
		un_str = c_char_p()
		ret = self._crypt32.CryptUnprotectData(byref(blob), byref(un_str),
						     None, None, None, 0, byref(blob2))
		if ret != 1:
			print "Error decrypting"
		username = string_at(un_str)
		password = string_at(blob2.pbData, blob2.cbData)

		return (username, password)
		
	def set_creds(self, service, config, username, password):
		blob = BLOB()
		blob2 = BLOB()
		blob.cbData = len(password)
		pass_str = c_char_p(password)
		blob.pbData = cast(pass_str, c_void_p)
		ret = self._crypt32.CryptProtectData(byref(blob), username,
						     None, None, None, 0, byref(blob2))
		if ret != 1:
			print "Error encrypting"
		bin_pw = repr(string_at(blob2.pbData, blob2.cbData))

		config[service]["bpassword"] = bin_pw
		
		return
		
	def __init__ (self):
		GKeyring.__init__(self)
		self._crypt32 = windll.crypt32
		



_keyring = None

def detect_keyring():
	if os.name == "nt":
		return GKeyringW32()
	else:
		if not _has_dbus:
			if _has_gkeyring:
				# Maybe check to see if it's running?
				return GKeyringGnome()
			return None
		

		# Why do we loop twice?  In case both are running, we default
		# to gnome this way.  Otherwise, we'd just pick a random one.		
		for i in _bus.list_names():
			if i == "org.gnome.keyring":
				return GKeyringGnome()
		for i in _bus.list_names():
			if i == "org.kde.kwalletd":
				return GKeyringKDE()

		# Nothing running.  Fall back to
		# just storing in plaintext.  We could try to get
		# dbus to start one up, but screw that.  If you want
		# keyrings, you ought to be running a key ring manager :)

	return None


def get_creds(service, backup=False):
	global _keyring
	if backup:
		config = GMain._GConf.backupconfig["global"]["accounts"]
	else:
		config = GMain._GConf.gconfig["global"]["accounts"]
	if _keyring == None:
		_keyring = detect_keyring()
	if _keyring == None:
		username = config[service]["username"]
		password = config[service]["password"]
		return (username, password)
		
	if config[service]["username"] != "KEYRING": 
		username = config[service]["username"]
		password = config[service]["password"]
		set_creds(service, username, password, backup)
		return (username, password)
		# Let's convert to the keyring.
		
	return _keyring.get_creds(service, config)


def set_creds(service, username, password, backup=False):
	global _keyring
	if backup:
		config = GMain._GConf.backupconfig["global"]["accounts"]
	else:
		config = GMain._GConf.gconfig["global"]["accounts"]
	
	if _keyring == None:
		_keyring = detect_keyring()

	if _keyring == None:
		config[service]["username"] = username
		config[service]["password"] = password
		if not backup:
			GMain._GConf.save_config()
		return

	_keyring.set_creds(service, config, username, password)
	config[service]["username"] = "KEYRING"
	config[service]["password"] = "KEYRING"
	if not backup:
		GMain._GConf.save_config()


	return
