# -*- coding: utf-8 -*-

import googsystray
import gobject
import gtk
import threading
import os

try:
	import pynotify
	pynotify.init("Googsystray")
	_has_pynotify = True
except:
	_has_pynotify = False
	

try:
	import PySnarl
	_has_pysnarl = True
except:
	_has_pysnarl = False
	

import GMain
import GPopupWindow

def start():
        if _has_pysnarl:
               	print PySnarl.snRegisterConfig2(GMain._hwnd, "googsystray", 0x12345, os.path.join(googsystray.BASE_DIR, "icons", "Google-64.png"))

def stop():
	if _has_pysnarl:
		PySnarl.snRevokeConfig(GMain._hwnd)


#########################################################
# Sound support
#########################################################
_player_lock = threading.Lock()

if os.name == "nt":
	from winsound import PlaySound, SND_FILENAME, SND_ASYNC
else:
	import wave
	import ossaudiodev
	import ctypes
	
	asound = None
	
	try:
		asound = ctypes.cdll.LoadLibrary("libasound.so.2")
		handle = ctypes.c_void_p()
		asound.snd_pcm_open(ctypes.byref(handle), "default", 0, 1)
	except:
		pass # fixme real error

def get_sound_file(fname):
	if fname[0] != '$':
		return fname
	return os.path.join(googsystray.BASE_DIR, "sounds", fname[1:])

def play_sound_linux_oss(snd_file):
	_player_lock.acquire()
	f = wave.open(snd_file, "rb")
	(nc,sw,fr,nf,comptype,compname) = f.getparams()
	d = ossaudiodev.open("/dev/dsp", "w")
	d.setparameters(ossaudiodev.AFMT_S16_LE, nc, fr)
	data = f.readframes(nf)
	f.close()
	d.write(data)
	d.close()
	
	_player_lock.release()
	
	
def play_sound_linux_alsa(snd_file):
	_player_lock.acquire()
	f = wave.open(snd_file, "rb")
	(nc,sw,fr,nf,comptype,compname) = f.getparams()
	params = ctypes.c_void_p()

	asound.snd_pcm_hw_params_malloc(ctypes.byref(params))
	asound.snd_pcm_hw_params_any(handle, params)
	asound.snd_pcm_hw_params_set_access(handle, params, 3)
	asound.snd_pcm_hw_params_set_format(handle, params, 2)
	asound.snd_pcm_hw_params_set_channels(handle, params, nc)
	asound.snd_pcm_hw_params_set_rate(handle, params, fr, 0)
	asound.snd_pcm_hw_params(handle, params)

	data = f.readframes(nf)
	f.close()
	asound.snd_pcm_prepare(handle)
	count = nf
	while count > 0:
		c = asound.snd_pcm_writei(handle, data, count)
		if c == -11:
			asound.snd_pcm_wait(handle, 500)
			continue
		if c == -32:
			asound.snd_pcm_prepare(handle)
			continue	
		data = data[c*2*nc:]
		count -= c
	_player_lock.release()

#credit Bill Dandreta Windows code
def play_sound(fname):
	snd_file = get_sound_file(fname)
	if os.name == "nt":
		PlaySound(snd_file, SND_FILENAME|SND_ASYNC)
	elif os.name == "posix":
		if asound is not None:
			threading.Thread(target=play_sound_linux_alsa, args=[snd_file]).start()
		else:
			threading.Thread(target=play_sound_linux_oss, args=[snd_file]).start()
	else:
		print "Sound not supported on this platform."






def get_config(type):
	return GMain._GConf.gconfig[type]
def get_backupconfig(type):
	return GMain._GConf.backupconfig[type]


def notify(from_icon, message, first):
	aname = message["account"]
	type = message["sub_id"]
	print message
	print first
	
	for i in get_config(message["type"])["sub_notifications"]:
		if i["account"] != aname:
			continue
		if i["webname"] == type:
			if first:
				if i["enable_sound"]:
					play_sound(i["sound_file"])
			if i["enable_other"]:
				GBase.run_command(i["other_command"])
			break

	
	if message["icon"] is not None:
		icon = message["icon"]
	else:
		icon = os.path.join(googsystray.BASE_DIR, "icons", get_config(message["type"])["icon"])

	if get_config("global")["vis_notify"] == "pynotify":
		n = pynotify.Notification(message["title"], message["summary"], icon)
		n.set_timeout(get_config("global")["vis_notify_timeout"] * 1000)
		n.set_urgency(pynotify.URGENCY_NORMAL)
		n.attach_to_status_icon(from_icon.staticon)
		n.show()
	elif get_config("global")["vis_notify"] == "native":
		n = pynotify.Notification(message["title"], message["summary"], icon)
		n.show()
	elif get_config("global")["vis_notify"] == "pysnarl":
		PySnarl.snShowMessage(message["title"], 
				      message["summary"], 
				      timeout=get_config("global")["vis_notify_timeout"], 
				      iconPath=icon)
	


notify_types = [
		{ "printname" : "None",
		  "name" : "none",
		  "has" : True,
		  "has_timeout" : False,
		  "has_number" : False,
		},
		{ "printname" : "Native",
		  "name" : "native",
		  "has" : True
		},
		{ "printname" : "PyNotify",
		  "name" : "pynotify",
		  "has" : _has_pynotify,
		},
		{ "printname" : "Snarl",
		  "name" : "snarl",
		  "has" : _has_pysnarl,
		},
	]

vis_notify = None
entries = []

def close_config(saved):
	return
	
def fix_notify(data=None):
	type = None
	model = vis_notify.get_model()
	for i in notify_types:
	        
		if _(i["printname"]) == model[vis_notify.get_active()][0]:
			type = i
			
			
	if type is None:
		return
		
	get_backupconfig("global")["vis_notify"] = type["name"]
	
		
		
	return
	
def draw_config(vbox):
	global vis_notify
	hbox = gtk.HBox()
	 
	label = gtk.Label(_("Visual Notification Type:"))
	
	hbox.pack_start(label, False, False, 5)
	
	vis_notify = gtk.combo_box_new_text()
	c=0
	for i in notify_types:
		if not i["has"]:
			continue
		vis_notify.append_text(_(i["printname"]))
		if i["name"] == get_config("global")["vis_notify"]:
			vis_notify.set_active(c)
		c+=1
	vis_notify.connect("changed", fix_notify)
		
	hbox.pack_start(vis_notify, False, False, 5)
	vbox.pack_start(hbox, False, False, 5)
