# -*- coding: utf-8 -*-

import googsystray
import gtk,gobject
import cairo
import pango
import pangocairo
import os
import types
import math
import GConf
import GMain

WINDOW_PADDING=32

GRADIENT_CENTERED = 0x01
GRADIENT_DIAG     = 0x02

GRADIENT_2	= 0x10000
GRADIENT_3	= 0x20000

popup_schemes = [
		{ "name": "blue",
		  "printname": _("Blue"),
		  "popup_bg_color1": "#3a5a84",
		  "popup_bg_color2": "#738aa7",
		  "popup_border_color": "#ced7e1",
		  "popup_font_color": "#000000",
		  "popup_gradient": (GRADIENT_CENTERED | GRADIENT_3),
		  "custom": False,
		},
		{ "name": "old_style",
		  "printname": _("Old_Style"),
		  "popup_bg_color1": "#A0A0FF",
		  "popup_bg_color2": "#FFFFFF",
		  "popup_border_color": "#000000",
		  "popup_font_color": "#000000",
		  "popup_gradient": (GRADIENT_DIAG | GRADIENT_2),
		  "custom": False,
		},
#		{ "name": "ubuntu",
#		  "printname": "Ubuntu",
#		  "popup_bg_color1": "#131313",
#		  "popup_bg_color2": "#131313",
#		  "popup_border_color": "#131313",
#		  "popup_font_color": "#FFFFFF",
#		  "custom": False,
#		},
		{ "name": "custom",
		  "printname": "Custom",
		  "popup_gradient": (GRADIENT_CENTERED | GRADIENT_3),
		  "custom": True,
		}
	]


def get_popup_scheme(name):
	for i in popup_schemes:
		if name == i["name"]:
			return i
	return None

def get_rgb(color):
	c = gtk.gdk.color_parse(color)
	r = float(c.red)/65535.0
	g = float(c.green)/65535.0
	b = float(c.blue)/65535.0
	return (r,g,b)


def get_color(color):
	scheme = get_popup_scheme(get_config()["popup_scheme"])

	if scheme["custom"]:
		c = get_config()[color]
	else:
		if type(scheme[color]) == types.StringType:
			c = scheme[color]
		else:
			# FIXME this will be for being able to use functions
			# to automatically pick up colors from the system
			printf("ERROR!!!")
	
	

	return get_rgb(c)

class GPopupWindow:

	def destroy_popup(self, widget, event, data=None):
		self.destroy()
	
	def create(self):
		return
		
	def destroy(self, data=None, data2=None):
		self.window.destroy()
		self.destroy_func()
		return False
		
	def show_all(self):
		self.window.show_all()

	def __init__(self, destroy_func, title, x=0xFFFF, y=0xFFFF, icon=None):
		self.origLocX = x
		self.origLocY = y
		self.title = title
		self.destroy_func = destroy_func
		self.window = None
		self.vbox = None
		
		self.create()




class GPopupWindowNative(GPopupWindow):
	def button_press(self, widget, event, data=None):
	 	(w,h) = widget.get_size()
		sw = self.shadow_width
		rsw = sw + self.radius
		if event.x < (w-rsw-self.close_size_w) or event.x > (w-sw):
			return
		if event.y < sw or event.y > rsw+self.close_size_h:
			return

		self.destroy()
		
		
	def reshape(self, obj, allocation):
		w,h = allocation.width, allocation.height

		# Ack!  When you change the contents of a label, it
		# reallocates its size.  Which of course means its parent
		# gets a reallocate called even if it didn't change.
		# Which furthermore means THIS function ends up getting
		# called on every single keypress in a GV window when typing
		# a text message.  Not healthy.
#		print "Reshape " + str(w) + str(h)

		if self.height == h and self.width == w:
			# We still need to relocate here.
			self.relocate(obj)
			return True

		rect = gtk.gdk.Rectangle(0,0,w,h)
		if obj.window is not None:
			obj.window.invalidate_rect(rect, True)
			

		self.height = h
		self.width = w


		# If we've got a composited WM, we'll handle this
		# with transparency instead.
		if obj.is_composited():
	                self.relocate(obj)
			return True			
			

		bitmap = gtk.gdk.Pixmap(None, w, h, 1)
		cr = bitmap.cairo_create()

		# Clear the bitmap
		cr.set_source_rgb(0.0,0.0,0.0)
		cr.set_operator(cairo.OPERATOR_CLEAR)
		cr.paint()
                
                cr.set_operator(cairo.OPERATOR_SOURCE)

		cr.set_source_rgb(1.0,1.0,1.0)
		
		cr.move_to(0+self.radius, 0)
		cr.line_to(w-self.radius, 0)
		cr.arc(w-self.radius, 0+self.radius, self.radius, math.pi/2, 0)
		cr.line_to(w, h-self.radius)
		cr.arc(w-self.radius, h-self.radius, self.radius, 0, math.pi/2)
		cr.line_to(0+self.radius, h)
		cr.arc(0+self.radius, h-self.radius, self.radius, math.pi+math.pi/2, math.pi)
		cr.line_to(0, 0+self.radius)
		cr.arc(0+self.radius, 0+self.radius, self.radius, math.pi/2, 0)

		cr.fill()

                """
                cr.set_operator(cairo.OPERATOR_CLEAR)
		cr.set_line_width(2.0)

		cr.set_source_rgb(0.0,0.0,0.0)
		cr.move_to(0+self.radius, h)
		cr.arc(0+self.radius, h-self.radius, self.radius, math.pi/2, -math.pi)
		cr.line_to(0, 0+self.radius)
		cr.arc(0+self.radius, 0+self.radius, self.radius, -math.pi, -math.pi/2)
		cr.line_to(w-self.radius, 0)
		cr.arc(w-self.radius, 0+self.radius, self.radius, -math.pi/2, 0)
		cr.stroke()
		"""
                
                # Set the window shape
                obj.shape_combine_mask(bitmap, 0, 0)

                self.relocate(obj)
                        


	def relocate(self, widget):
		(winWidth, winHeight) = widget.get_size()

#		print "Relocate " + str(winWidth) + str(winHeight)

		
		screen = widget.get_screen()

                # This is a _total_ hack.  Ultimately I need to be
                # using label names and GtkRC and whatnot.
		screen.set_resolution(96.0*self.size)

		if self.origLocX == 0xFFFF:
			# Opens @ the center of whatever monitor the mouse pointer is
			# currently on.  This is probably the sanest way to do this.
			
			# FIXME test if this might break on multi-SCREEN systems. 
			# I think I might be only able to put up windows on the screen
			# the staticon is on, although this here should work for xinerama and
			# nview
                        (m_x, m_y, m_mask) = widget.get_screen().get_root_window().get_pointer()
                        geometry = screen.get_monitor_geometry(screen.get_monitor_at_point(m_x, m_y))
                        scrWidth = geometry.width
                        scrHeight = geometry.height
			p_x = geometry.x + (scrWidth - winWidth)/2
			p_y = geometry.y + (scrHeight - winHeight)/2

			widget.move(p_x,p_y)
			return


		p_x = self.origLocX
		p_y = self.origLocY

		geometry = screen.get_monitor_geometry(screen.get_monitor_at_point(p_x, p_y))

		if p_x < (geometry.x + WINDOW_PADDING):
			p_x = geometry.x + WINDOW_PADDING
		if (p_x + winWidth) > ((geometry.x + geometry.width) - WINDOW_PADDING):
			p_x = (geometry.x + geometry.width) - WINDOW_PADDING - winWidth
		if p_y < (geometry.y + WINDOW_PADDING):
			p_y = geometry.y + WINDOW_PADDING
		if (p_y + winHeight) > ((geometry.y + geometry.height) - WINDOW_PADDING):
			p_y = (geometry.y + geometry.height) - WINDOW_PADDING - winHeight

		widget.move(p_x,p_y)
		
	def expose(self, widget, event, data=None):
		cr = widget.window.cairo_create()
		(w,h) = widget.get_size()
		
#		print "Expose " + str(w) + str(h)

		cr.rectangle(event.area.x, event.area.y,
		             event.area.width, event.area.height)
		cr.clip()
		
		cr.set_antialias(cairo.ANTIALIAS_GRAY)

		if widget.is_composited():
			cr.set_operator(cairo.OPERATOR_SOURCE)
			cr.set_source_rgba(1.0,1.0,1.0,0.0)
			cr.paint()

		# The below lines won't actually cause a redraw 
		# cause they'll be clipped, but cairo will still have
		# to do all the math.  So we'll cheap out here if it's
		# a redraw nowhere near the edges

		if get_config()["popup_dshadow"]:
			draw_edge = True
		else:
			draw_edge = False

		if (event.area.x > (self.radius + self.shadow_width) and event.area.y > (self.radius + self.shadow_width) and
		    (event.area.x+event.area.width) < (w-(self.radius + self.shadow_width)) and 
		    (event.area.y+event.area.height) < (h-(self.radius + self.shadow_width))):
#		    	print "No edge"
		    	draw_edge = False
		    	
		    					
		cr.set_operator(cairo.OPERATOR_SOURCE)

		sw = self.shadow_width
		sw2 = sw/2
		rsw2 = sw2 + self.radius
		rsw = sw + self.radius
		


		if draw_edge:
			strength = get_config()["popup_dshadow_str"]*0.75;

			cr.set_source_rgba(0.0, 0.0, 0.0, strength)

			cr.rectangle(sw,sw,w-sw*2,h-sw*2)
			cr.fill();
			
			cr.set_line_width(self.shadow_width)


			lg = cairo.LinearGradient(0, 0, self.shadow_width, 0)
			lg.add_color_stop_rgba(0.0, 0, 0, 0, 0.0)
			lg.add_color_stop_rgba(1.0, 0, 0, 0, strength) 
			cr.set_source(lg)
			cr.move_to(sw2, h-rsw2)
			cr.line_to(sw2, rsw2)
			cr.stroke()



			lg = cairo.LinearGradient(0, 0, 0, self.shadow_width)
			lg.add_color_stop_rgba(0.0, 0, 0, 0, 0.0)
			lg.add_color_stop_rgba(1.0, 0, 0, 0, strength) 
			cr.set_source(lg)
			cr.move_to(rsw2, sw2)
			cr.line_to(w-rsw2, sw2)
			cr.stroke()


			lg = cairo.LinearGradient(w, 0, w-self.shadow_width, 0)
			lg.add_color_stop_rgba(0.0, 0, 0, 0, 0.0)
			lg.add_color_stop_rgba(1.0, 0, 0, 0, strength) 
			cr.set_source(lg)
			cr.move_to(w-sw2, rsw2)
			cr.line_to(w-sw2, h-rsw2)
			cr.stroke()

			lg = cairo.LinearGradient(0, h, 0, h-self.shadow_width)
			lg.add_color_stop_rgba(0.0, 0, 0, 0, 0.0)
			lg.add_color_stop_rgba(1.0, 0, 0, 0, strength) 
			cr.set_source(lg)
			cr.move_to(w-rsw2, h-sw2)
			cr.line_to(rsw2, h-sw2)
			cr.stroke()

			rg = cairo.RadialGradient(rsw, h-rsw, self.radius,
						  rsw, h-rsw, rsw)
			rg.add_color_stop_rgba(0.0, 0, 0, 0, strength)
			rg.add_color_stop_rgba(1.0, 0, 0, 0, 0.0) 
			cr.set_source(rg)
			cr.move_to(rsw2, h-sw2)
			cr.arc(rsw, h-rsw, rsw2, math.pi/2, -math.pi)
			cr.stroke()

			rg = cairo.RadialGradient(rsw, rsw, self.radius,
						  rsw, rsw, rsw)
			rg.add_color_stop_rgba(0.0, 0, 0, 0, strength)
			rg.add_color_stop_rgba(1.0, 0, 0, 0, 0.0) 
			cr.set_source(rg)
			cr.move_to(sw2, rsw2)
			cr.arc(rsw, rsw, rsw2, -math.pi, -math.pi/2)
			cr.stroke()


			rg = cairo.RadialGradient(w-rsw, rsw, self.radius,
						  w-rsw, rsw, rsw)
			rg.add_color_stop_rgba(0.0, 0, 0, 0, strength)
			rg.add_color_stop_rgba(1.0, 0, 0, 0, 0.0) 
			cr.set_source(rg)
			cr.move_to(w-rsw2, sw2)
			cr.arc(w-rsw, rsw, rsw2, -math.pi/2, 0)
			cr.stroke()


			rg = cairo.RadialGradient(w-rsw, h-rsw, self.radius,
						  w-rsw, h-rsw, rsw)
			rg.add_color_stop_rgba(0.0, 0, 0, 0, strength)
			rg.add_color_stop_rgba(1.0, 0, 0, 0, 0.0) 
			cr.set_source(rg)
			cr.move_to(w-sw2, h-rsw2)
			cr.arc(w-rsw, h-rsw, rsw2, 0, math.pi/2)
			cr.stroke()

                cr.set_operator(cairo.OPERATOR_SOURCE)
		cr.set_line_width(2.0)
		cr.set_source_rgba(self.border_color[0],
		                   self.border_color[1],
		                   self.border_color[2],
		                   get_config()["popup_opacity"])
		
		if get_config()["popup_dshadow"]:
			cr.translate(-1,-1)
		
		cr.move_to(rsw+1, h-sw-1)
		cr.arc(rsw+1, h-rsw-1, self.radius, math.pi/2, -math.pi)
		cr.line_to(sw+1, rsw+1)
		cr.arc(rsw+1, rsw+1, self.radius, -math.pi, -math.pi/2)
		cr.line_to(w-rsw-1, sw+1)
		cr.arc(w-rsw-1, rsw+1, self.radius, -math.pi/2, 0)
		cr.line_to(w-sw-1, h-rsw-1)
		cr.arc(w-rsw-1, h-rsw-1, self.radius, 0, math.pi/2)
		cr.line_to(rsw+1, h-sw-1)
		cr.stroke_preserve()

		grad = get_popup_scheme(get_config()["popup_scheme"])["popup_gradient"]
		
		if grad & GRADIENT_DIAG:
			lg1 = cairo.LinearGradient(0, 0, w, h)
		elif grad & GRADIENT_CENTERED:
			lg1 = cairo.LinearGradient(0, 0, w, 0)
			
			
		if grad & GRADIENT_2:
			lg1.add_color_stop_rgba(0.0, self.bg1_color[0], 
			                             self.bg1_color[1], 
			                             self.bg1_color[2], 
			                             get_config()["popup_opacity"])
			lg1.add_color_stop_rgba(1.0, self.bg2_color[0], 
			                             self.bg2_color[1], 
		        	                     self.bg2_color[2], 
		                	             get_config()["popup_opacity"])
		elif grad & GRADIENT_3:
			lg1.add_color_stop_rgba(0.0, self.bg1_color[0], 
			                             self.bg1_color[1], 
			                             self.bg1_color[2], 
			                             get_config()["popup_opacity"])
			lg1.add_color_stop_rgba(0.5, self.bg2_color[0], 
			                             self.bg2_color[1], 
		        	                     self.bg2_color[2], 
		                	             get_config()["popup_opacity"])
			lg1.add_color_stop_rgba(1.0, self.bg1_color[0], 
			                             self.bg1_color[1], 
			                             self.bg1_color[2], 
		        	                     get_config()["popup_opacity"])
		cr.set_source(lg1)
		
		cr.fill()		

		"""
                cr.set_operator(cairo.OPERATOR_SOURCE)
                cr.set_antialias(cairo.ANTIALIAS_NONE)
		cr.set_line_width(1.0)
		cr.set_source_rgba(self.border_color[0],
		                   self.border_color[1],
		                   self.border_color[2],
		                   get_config()["popup_opacity"])
		cr.move_to(w-rsw-self.close_size_w, sw)
		cr.line_to(w-rsw-self.close_size_w, rsw+self.close_size_h)
		cr.line_to(w-sw, rsw+self.close_size_h)
		cr.stroke()
		cr.set_antialias(cairo.ANTIALIAS_GRAY)
		"""
		
		fo = cairo.FontOptions()
		fm = pangocairo.cairo_font_map_get_default()
		fm.set_resolution(96.0*self.size)
		fo.set_antialias(cairo.ANTIALIAS_GRAY)
		
		pc = pangocairo.CairoContext(cr)
		pc.set_font_options(fo)
		pc.set_operator(cairo.OPERATOR_SOURCE)
		pc.set_line_join(cairo.LINE_JOIN_ROUND)

		layout = pc.create_layout()
		
		pc.set_antialias(cairo.ANTIALIAS_GRAY)
		layout.set_font_description(pango.FontDescription("sans bold 10"))

		layout.set_text(self.title)
		
		mv = 0
		
		if self.icon_pb is not None:
			(lw,lh) = layout.get_pixel_size()
			mv = lh + 5
		
			if self.icon_pb.get_height() != lh:
				self.icon_pb = self.icon_pb.scale_simple(lh,lh,gtk.gdk.INTERP_BILINEAR)
				
			widget.window.draw_pixbuf(None,
			                          self.icon_pb,
			                          0,0,
			                          rsw,sw+1)
			                          
			
			
#		widget.window.draw_pixbuf(None, self.close_pb, 0, 0, w-sw-self.close_pb.get_width()-1, sw-1)
		
		
		
		pc.move_to(rsw+mv,sw+1)
		pc.layout_path(layout)
		pc.set_line_width(2.0)
		pc.set_source_rgb(0,0,0)
		pc.stroke_preserve()
		
		pc.set_source_rgb(1,1,1)
		pc.fill()
		
		
		layout.set_text("x")
		pc.set_source_rgb(0,0,0)
		pc.move_to(w-rsw-self.close_size_w, sw+1)
		pc.layout_path(layout)
		pc.stroke_preserve()
		pc.set_source_rgb(1,1,1)
		pc.fill()

		return False

	def button_release(self, window, event, data=None):
		self.old_x = -1
		self.old_y = -1

	def pointer_motion(self, window, event, data=None):
		if event.state & gtk.gdk.BUTTON1_MASK:
			if self.old_x != -1:
				print event.x
				diff_x = event.x_root - self.old_x
				diff_y = event.y_root - self.old_y
				(x,y) = self.window.get_position()
				print diff_x
				print x
				x += diff_x
				y += diff_y
				print x
				print "-----"
				self.window.move(x,y)
			
			
			self.old_x = event.x_root
			self.old_y = event.y_root
		else:
			self.old_x = -1
			self.old_y = -1
		
	def create(self):
		self.size = get_config()["popup_size"]
		self.close_size_h = 2
		self.close_size_w = 5

		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_decorated(False)
		if get_config()["popup_ontop"]:
			self.window.set_keep_above(True)
		if get_config()["popup_sticky"]:
			self.window.stick()
#		self.window.set_default_size(480 + self.shadow_width*2, -1)
		if get_config()["popup_dshadow"] and self.window.is_composited():
			self.shadow_width = 2 * get_config()["popup_dshadow_size"];
		else:
			self.shadow_width = 0;
		
		self.window.set_app_paintable(True)
		colormap = self.window.get_screen().get_rgba_colormap()
		if colormap == None:
			colormap = self.window.get_screen().get_rgb_colormap()
		gtk.widget_set_default_colormap(colormap)
	
		self.vbox = gtk.VBox()
		# This hbox is simply to provide a window border.  As with
		# most of the things in here, probably a better way to do it :)
		self.align = gtk.Alignment(0.0, 0.0, 1.0, 1.0)
		self.window.add(self.align)
		self.align.add(self.vbox);

		if get_config()["popup_dshadow"]:
			right_padding = 4
		else:
			right_padding = 0
		self.align.set_padding(self.shadow_width + 2, self.shadow_width + 6, self.shadow_width + 2, self.shadow_width + right_padding)
		
		hbox = gtk.HBox()
		self.vbox.pack_start(hbox, False, False, 0)
		
		
		label = gtk.Label()
		#label.modify_text(gtk.STATE_NORMAL, gtk.gdk.color_parse(get_config()["popup_font_color"]))
		#label.set_markup("<b><u>" + self.title + "</u></b>")
		hbox.pack_start(label, True, True, 5)
##		ebox = gtk.EventBox()
#		ebox.set_visible_window(False)
		self.window.set_events(gtk.gdk.BUTTON_PRESS_MASK |
				       gtk.gdk.BUTTON_MOTION_MASK |
				       gtk.gdk.BUTTON_RELEASE_MASK)
		self.window.connect("button_press_event", self.button_press)
		self.window.connect("button_release_event", self.button_release)
		self.window.connect("motion_notify_event", self.pointer_motion)

#		ebox.add(image)
#		hbox.pack_end(image, False, False, 0)
		
		self.relocate(self.window)
		self.window.connect("expose-event", self.expose)
		self.window.connect("size-allocate", self.reshape)
		#self.window.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))

		if not self.window.is_composited():
        		self.window.set_opacity(get_config()["popup_opacity"])

	def __init__(self, destroy_func, title, x=0xFFFF, y=0xFFFF, icon=None):
		self.width = -1
		self.height = -1
		self.radius = 12
		self.old_x = -1
		self.old_y = -1
		
		self.resized = False
		

		if icon is not None:
			self.icon_pb = gtk.gdk.pixbuf_new_from_file(icon)
		else:
			self.icon_pb = None

		self.bg1_color = get_color("popup_bg_color1")
		self.bg2_color = get_color("popup_bg_color2")
		self.border_color = get_color("popup_border_color")
		self.font_color = get_color("popup_border_color")

		GPopupWindow.__init__(self, destroy_func, title, x, y, icon)

class GPopupWindowClean(GPopupWindowNative):
	def mouse_leave(self, window, event, data=None):
		if event.x < 0 or event.y < 0 or event.x > self.width or event.y > self.height:
			self.destroy()

	def mouse_enter(self, window, data=None):
		pass

	def create(self):
		GPopupWindowNative.create(self)
		self.window.set_events(gtk.gdk.LEAVE_NOTIFY_MASK |
				       gtk.gdk.ENTER_NOTIFY_MASK |
				       gtk.gdk.BUTTON_PRESS_MASK)
		self.window.connect("leave_notify_event", self.mouse_leave)
		self.window.connect("enter_notify_event", self.mouse_enter)

	
	def __init__(self, destroy_func, title, x=0xFFFF, y=0xFFFF, icon=None):
		GPopupWindowNative.__init__(self, destroy_func, title, x, y, icon)
		self.radius = 6
		self.bg1_color = get_rgb("#FFFFFF")
		self.bg2_color = get_rgb("#FFFFFF")
		self.border_color = get_rgb("#000000")


class GPopupWindowGtk(GPopupWindow):
	def create(self):
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		if get_config()["popup_ontop"]:
			self.window.set_keep_above(True)
		if get_config()["popup_sticky"]:
			self.window.stick()
#		self.window.set_default_size(480, -1)
		self.window.set_title(self.title)

		self.vbox = gtk.VBox()
		self.window.add(self.vbox)
		

	def __init__(self, destroy_func, title, x=0xFFFF, y=0xFFFF, icon=None):
		GPopupWindow.__init__(self, destroy_func, title, x, y, icon)

		
popup_types = [
		{ "name": "native",
		  "printname": _("Native"),
		  "obj": GPopupWindowNative,
		  "uses_scheme": True
		},
		{ "name": "gtk",
		  "printname": _("Default GTK (Ignores many settings)"),
		  "obj": GPopupWindowGtk,
		  "uses_scheme": False
		},
	]


use_backup = False

def get_config():
	if use_backup:
		return GMain._GConf.backupconfig["global"]
	
	return GMain._GConf.gconfig["global"]




def close_config(saved, fake=False):
	global config_vbox
	global sticky_cbutton, ontop_cbutton, dshadow_cbutton
	global popup_type
	global opacity_adj, size_adj, dshadow_size_adj, dshadow_str_adj
	
	if saved:
		GMain._GConf.backupconfig["global"]["popup_opacity"] = opacity_adj.get_value()/100.0
		GMain._GConf.backupconfig["global"]["popup_size"] = size_adj.get_value()/100.0
		GMain._GConf.backupconfig["global"]["popup_sticky"] = sticky_cbutton.get_active()
		GMain._GConf.backupconfig["global"]["popup_ontop"] = ontop_cbutton.get_active()
		GMain._GConf.backupconfig["global"]["popup_style"] = popup_types[popup_type.get_active()]["name"]
		GMain._GConf.backupconfig["global"]["popup_scheme"] = popup_schemes[popup_scheme.get_active()]["name"]
		
		if popup_schemes[popup_scheme.get_active()]["custom"]:
			GMain._GConf.backupconfig["global"]["popup_bg_color1"] = color1_button.get_color().to_string()
			GMain._GConf.backupconfig["global"]["popup_bg_color2"] = color2_button.get_color().to_string()
			GMain._GConf.backupconfig["global"]["popup_border_color"] = color3_button.get_color().to_string()
			GMain._GConf.backupconfig["global"]["popup_font_color"] = color4_button.get_color().to_string()

                if os.name == "nt":
                        GMain._GConf.backupconfig["global"]["popup_dshadow"] = False
                else:
                        GMain._GConf.backupconfig["global"]["popup_dshadow_str"] = dshadow_str_adj.get_value()/100.0
                        GMain._GConf.backupconfig["global"]["popup_dshadow_size"] = int(dshadow_size_adj.get_value())
                        GMain._GConf.backupconfig["global"]["popup_dshadow"] = dshadow_cbutton.get_active()
	
	if not fake:
		config_vbox = None
	return

def fix_schemes(widget=None, data=None):
	popup_scheme.set_sensitive(False)
	popup_scheme_label.set_sensitive(False)
	if popup_types[popup_type.get_active()]["uses_scheme"]:
		popup_scheme_label.set_sensitive(True)
		popup_scheme.set_sensitive(True)

def fix_colors(widget=None, data=None):
	color1_label.set_sensitive(False)
	color1_button.set_sensitive(False)
	color2_label.set_sensitive(False)
	color2_button.set_sensitive(False)
	color3_label.set_sensitive(False)
	color3_button.set_sensitive(False)
	color4_label.set_sensitive(False)
	color4_button.set_sensitive(False)
	copy_custom_button.set_sensitive(False)

	if popup_schemes[popup_scheme.get_active()]["custom"]:
		color1_label.set_sensitive(True)
		color1_button.set_sensitive(True)
		color2_label.set_sensitive(True)
		color2_button.set_sensitive(True)
		color3_label.set_sensitive(True)
		color3_button.set_sensitive(True)
		color4_label.set_sensitive(True)
		color4_button.set_sensitive(True)

		c1 = gtk.gdk.color_parse(get_config()["popup_bg_color1"])
		c2 = gtk.gdk.color_parse(get_config()["popup_bg_color2"])
		c3 = gtk.gdk.color_parse(get_config()["popup_border_color"])
		c4 = gtk.gdk.color_parse(get_config()["popup_font_color"])
		
	else:
		c1 = gtk.gdk.color_parse(popup_schemes[popup_scheme.get_active()]["popup_bg_color1"])
		c2 = gtk.gdk.color_parse(popup_schemes[popup_scheme.get_active()]["popup_bg_color2"])
		c3 = gtk.gdk.color_parse(popup_schemes[popup_scheme.get_active()]["popup_border_color"])
		c4 = gtk.gdk.color_parse(popup_schemes[popup_scheme.get_active()]["popup_font_color"])
		
		copy_custom_button.set_sensitive(True)
	
	color1_button.set_color(c1)
	color2_button.set_color(c2)
	color3_button.set_color(c3)
	color4_button.set_color(c4)
	
	
		
def test_destroy(widget=None, data=None):
	global test_window
	use_backup = False
	test_window = None
	
def test_popup(widget=None, data=None):
	global test_window
	global use_backup
	use_backup = True
	close_config(True, True)
	test_window = get_new_popup_window(test_destroy, 
	                                   _("Popup Window Test"),
	                                   icon=os.path.join(googsystray.BASE_DIR, "icons", get_config()["icon"]))
	test_window.window.set_modal(True)
	
	
	vbox = test_window.vbox

	# sep = gtk.HSeparator()
	# vbox.pack_start(sep, False, False, 3)

	fromt_label = gtk.Label()
	fromt_label.set_markup("<span color=\"#ffffff\"><b>" + _("From:") + "</b></span>")
	subjt_label = gtk.Label()
	subjt_label.set_markup("    <b>" + _("Subject:") + "</b>")
	from_label = gtk.Label(_("Nobody"))
	subj_label = gtk.Label(_("Hello, there!"))
	time_label = gtk.Label()
	time_label.set_markup("  <span color=\"#404040\">4 Hours Ago</span>")
	label_label = gtk.Label()
	label_label.set_markup("<span color=\"#404040\"><i>(Inbox)</i></span>")

	hbox1 = gtk.HBox()
	hbox2 = gtk.HBox()
	hbox3 = gtk.HBox()
	vbox.pack_start(hbox1, False, False, 2)
	vbox.pack_start(hbox2, False, False, 2)
	vbox.pack_start(hbox3, False, False, 5)
		
	hbox1.pack_start(fromt_label, False, False, 2)
	hbox1.pack_start(from_label, False, False, 2)
	hbox1.pack_start(time_label, False, False, 2)
	hbox1.pack_end(label_label, False, False, 2)
	hbox2.pack_start(subjt_label, False, False, 2)
	hbox2.pack_start(subj_label, False, False, 2)

	label=gtk.Label()
	label.set_markup("<span color=\"#0000FF\" underline=\"single\">" + (_("Delete")) + "</span>")
	hbox3.pack_end(label, True, True, 10)
	label=gtk.Label()
	label.set_markup("<span color=\"#0000FF\" underline=\"single\">" + (_("Mark as read")) + "</span>")
	hbox3.pack_end(label, True, True, 10)
	label=gtk.Label()
	label.set_markup("<span color=\"#0000FF\" underline=\"single\">" + (_("Mark as spam")) + "</span>")
	hbox3.pack_end(label, True, True, 10)
	
	
	test_window.window.show_all()
	
	
def copy_colors(widget=None, data=None):
	c1 = popup_schemes[popup_scheme.get_active()]["popup_bg_color1"]
	c2 = popup_schemes[popup_scheme.get_active()]["popup_bg_color2"]
	c3 = popup_schemes[popup_scheme.get_active()]["popup_border_color"]
	c4 = popup_schemes[popup_scheme.get_active()]["popup_font_color"]

	GMain._GConf.backupconfig["global"]["popup_bg_color1"] = c1
	GMain._GConf.backupconfig["global"]["popup_bg_color2"] = c2
	GMain._GConf.backupconfig["global"]["popup_border_color"] = c3
	GMain._GConf.backupconfig["global"]["popup_font_color"] = c4

		
	

def draw_config(vbox):
	global sens_widgets, entries, sound_buttons, config_vbox
	sens_widgets = []
	entries = []
	sound_buttons = []
	config_vbox = vbox
	
	
	hbox = gtk.HBox()
	vbox.pack_start(hbox, False, False, 5)
	label = gtk.Label(_("Popup Window Opacity:"))
	label.set_tooltip_text(_("The opacity of the popup window.  On Linux systems, this requires a compositing window manager."))
	hbox.pack_start(label, False, False, 5)
	
	global opacity_adj, size_adj, dshadow_size_adj, dshadow_str_adj
	
	opacity_adj = gtk.Adjustment(get_config()["popup_opacity"]*100.0, 0.0, 100.0, 0.05, 0.05, 0.05)
	hslid = gtk.HScale(opacity_adj)
	hslid.set_increments(0.05, 0.05)
	hbox.pack_start(hslid, True, True, 15)
	
	hbox = gtk.HBox()
	vbox.pack_start(hbox, False, False, 5)
	label = gtk.Label(_("Popup Window Size:"))
	label.set_tooltip_text(_("The relative size of the popup window."))
	hbox.pack_start(label, False, False, 5)
	
	size_adj = gtk.Adjustment(get_config()["popup_size"]*100.0, 50.0, 100.0, 0.05, 0.05, 0.05)
	hslid = gtk.HScale(size_adj)
	hslid.set_increments(0.05, 0.05)
	hbox.pack_start(hslid, True, True, 15)


	if os.name != "nt":
        	frame = gtk.Frame(_("Drop Shadows"))
        	vbox.pack_start(frame, False, False, 5)
        	vbox1 = gtk.VBox()
        	frame.add(vbox1)
	
        	global dshadow_cbutton
        	dshadow_cbutton = gtk.CheckButton(_("Enable Drop Shadow"))
        	dshadow_cbutton.set_tooltip_text(_("Popups should have a drop shadow.  Only works with compositing window managers"))
        	dshadow_cbutton.set_active(get_config()["popup_dshadow"])
        	vbox1.pack_start(dshadow_cbutton, False, False, 2)

        	hbox = gtk.HBox()
        	vbox1.pack_start(hbox, False, False, 2)
        	label = gtk.Label(_("Drop Shadow Strength:"))
        	label.set_tooltip_text(_("The strength of the drop shadow."))
        	hbox.pack_start(label, False, False, 5)
	
        	dshadow_str_adj = gtk.Adjustment(get_config()["popup_dshadow_str"]*100.0, 0.0, 100.0, 0.05, 0.05, 0.05)
        	hslid = gtk.HScale(dshadow_str_adj)
        	hslid.set_increments(0.05, 0.05)
        	hbox.pack_start(hslid, True, True, 15)

        	hbox = gtk.HBox()
        	vbox1.pack_start(hbox, False, False, 2)
        	label = gtk.Label(_("Drop Shadow Size:"))
        	label.set_tooltip_text(_("The size of the drop shadow."))
        	hbox.pack_start(label, False, False, 5)
	
        	dshadow_size_adj = gtk.Adjustment(get_config()["popup_dshadow_size"], 0, 11, 1, 1, 1)
        	hslid = gtk.HScale(dshadow_size_adj)
        	hslid.set_digits(0)
        	hslid.set_increments(1, 1)
                hbox.pack_start(hslid, True, True, 15)


	
	
	global sticky_cbutton
	sticky_cbutton = gtk.CheckButton(_("Sticky Popups"))
	sticky_cbutton.set_tooltip_text(_("Popups should appear on all desktops (no effect in Windows)"))
	sticky_cbutton.set_active(get_config()["popup_sticky"])
	vbox.pack_start(sticky_cbutton, False, False, 5)

	global ontop_cbutton
	ontop_cbutton = gtk.CheckButton(_("Popups Always on Top"))
	ontop_cbutton.set_tooltip_text(_("Popups should appear on top of all other windows."))
	ontop_cbutton.set_active(get_config()["popup_ontop"])
	vbox.pack_start(ontop_cbutton, False, False, 5)

	hbox = gtk.HBox()
	label = gtk.Label(_("Popup Type: "))
	hbox.pack_start(label, False, False, 5)
	
	global popup_type
	popup_type = gtk.combo_box_new_text()
	c=0
	for i in popup_types:
		popup_type.append_text(_(i["printname"]))
		if i["name"] == get_config()["popup_style"]:
			popup_type.set_active(c)
		c+=1
	popup_type.connect("changed", fix_schemes)
		
	hbox.pack_start(popup_type, False, False, 5)
	vbox.pack_start(hbox, False, False, 5)

	global popup_scheme, popup_scheme_label

	hbox = gtk.HBox()
	popup_scheme_label = gtk.Label(_("Popup Scheme: "))
	hbox.pack_start(popup_scheme_label, False, False, 5)
	

	popup_scheme = gtk.combo_box_new_text()
	c=0
	for i in popup_schemes:
		popup_scheme.append_text(_(i["printname"]))
		if i["name"] == get_config()["popup_scheme"]:
			popup_scheme.set_active(c)
		c+=1
	popup_scheme.connect("changed", fix_colors)
		
	hbox.pack_start(popup_scheme, False, False, 5)
	vbox.pack_start(hbox, False, False, 5)

	if popup_schemes[popup_scheme.get_active()]["custom"]:
		c1 = gtk.gdk.color_parse(get_config()["popup_bg_color1"])
		c2 = gtk.gdk.color_parse(get_config()["popup_bg_color2"])
		c3 = gtk.gdk.color_parse(get_config()["popup_border_color"])
		c4 = gtk.gdk.color_parse(get_config()["popup_font_color"])
	else:
		c1 = gtk.gdk.color_parse(popup_schemes[popup_scheme.get_active()]["popup_bg_color1"])
		c2 = gtk.gdk.color_parse(popup_schemes[popup_scheme.get_active()]["popup_bg_color2"])
		c3 = gtk.gdk.color_parse(popup_schemes[popup_scheme.get_active()]["popup_border_color"])
		c4 = gtk.gdk.color_parse(popup_schemes[popup_scheme.get_active()]["popup_font_color"])


	global color1_label, color1_button
	color1_label = gtk.Label(_("BG Color #1:"))
	color1_button = gtk.ColorButton(c1)
	color1_button.has_alpha = False
	color1_button.set_title(_("Select BG Color #1"))
	hbox.pack_end(color1_button, False, False, 5)
	hbox.pack_end(color1_label, False, False, 5)
	
	global color2_label, color2_button, copy_custom_button
	hbox = gtk.HBox()
	copy_custom_button = gtk.Button(_("Copy Colors to Custom"))
	copy_custom_button.connect("clicked", copy_colors)
	hbox.pack_start(copy_custom_button, False, False, 5)
	color2_label = gtk.Label(_("BG Color #2:"))
	color2_button = gtk.ColorButton(c2)
	color2_button.has_alpha = False
	color2_button.set_title(_("Select BG Color #2"))
	hbox.pack_end(color2_button, False, False, 5)
	hbox.pack_end(color2_label, False, False, 5)
	vbox.pack_start(hbox, False, False, 5)


	global color3_label, color3_button
	hbox = gtk.HBox()
	button = gtk.Button(_("Test Popup"))
	button.connect("clicked", test_popup)
	hbox.pack_start(button, False, False, 5)
	color3_label = gtk.Label(_("Border Color:"))
	color3_button = gtk.ColorButton(c3)
	color3_button.has_alpha = False
	color3_button.set_title(_("Select Border Color #3"))
	hbox.pack_end(color3_button, False, False, 5)
	hbox.pack_end(color3_label, False, False, 5)
	vbox.pack_start(hbox, False, False, 5)

	global color4_label, color4_button
	hbox = gtk.HBox()
	color4_label = gtk.Label(_("Font Color:"))
	color4_button = gtk.ColorButton(c4)
	color4_button.has_alpha = False
	color4_button.set_title(_("Select Font Color"))
	hbox.pack_end(color4_button, False, False, 5)
	hbox.pack_end(color4_label, False, False, 5)
	vbox.pack_start(hbox, False, False, 5)


	fix_schemes()
	fix_colors()


def get_new_popup_window(destroy_func, title, x=0xFFFF, y=0xFFFF, icon=None):
	for i in popup_types:
		if i["name"] == get_config()["popup_style"]:
			return i["obj"](destroy_func, title, x, y, icon)
	else:
		return GPopupWindowGtk(destroy_func, title, x, y, icon)


def get_new_popup_window_clean(destroy_func, title, x=0xFFFF, y=0xFFFF, icon=None):
	return GPopupWindowClean(destroy_func, title, x, y, icon)


