# -*- coding: utf-8 -*-

import gtk

from xml.etree import ElementTree as ET

import GIcon			
import GMain

class GReader(GIcon.GIcon):
	def draw_config(self, vbox):
		GIcon.GIcon.draw_config(self,vbox)

		hbox = gtk.HBox()
		vbox.pack_start(hbox, False, False, 2)
		label = gtk.Label(_("Minimum unread to issue a notification: "))
		label.set_tooltip_text(_("Googsystray will not issue notifications until there are at least this many unread news items."))
		adj = gtk.Adjustment(GMain._GConf.backupconfig[self.config]["minimum_unread"], 1.0, 1000.0, 1.0, 10.0, 0.0)
		spin = gtk.SpinButton(adj, 0, 0)
		adj.connect("value_changed", self.spinner_changed, "minimum_unread")
		self.sens_widgets.append(label)
		self.sens_widgets.append(spin)
		label.set_data("make_sensitive", 1)
		spin.set_data("make_sensitive", 1)
		
		
		
		hbox.pack_start(label, False, False, 5)
		hbox.pack_start(spin, False, False, 5)

		check = gtk.CheckButton(_("Only notify once after passing minimum threshold."))
		check.set_tooltip_text(_("Googsystray will notify only once when it passes the minimum threshold for new items."))
		check.connect("toggled", self.check_toggled, "minimum_notify_once")
		self.sens_widgets.append(check)
		check.set_data("make_sensitive", 1)
		check.set_active(GMain._GConf.backupconfig[self.config]["minimum_notify_once"])
		vbox.pack_start(check, False, False, 5)



	def icon_check_data(self, account):
		self.prev_last_unread = self.last_unread
	
		data = self.urlopen2(account, 'https://www.google.com/reader/api/0/unread-count')
		if data == None:
			return False
		
		node = ET.XML(data)
		
		total_unread = 0
		text_desc = ""

		node = node.find("list")
		for o in node.findall("object"):
			fname = o.find("string").text[25:]
			if (fname == "/state/com.google/reading-list"):
				for n in o.findall("number"):
					if (n.attrib["name"] == "count"):
						total_unread = int(n.text)
					if (n.attrib["name"] == "newestItemTimestampUsec"):
						self.last_unread = long(n.text)
			elif self.get_config()["show_by_label"]:
				if (fname[:7] == "/label/"):
					label = fname[7:]
					total = 0
					for n in o.findall("number"):
						if (n.attrib["name"] == "count"):
							total = int(n.text)
					text_desc += label + " (" + str(total) + " " + _("unread") + ")\n"
					
		text_desc += "Total: " + str(total_unread) + " unread."

		new_messages = [
				{ "id": "reader" + str(total_unread),
				  "sub_id": "article",
				  "account": account["name"],
				  "count": total_unread,
				  "title": _("New Google Reader Articles"),
				  "summary": text_desc,
				  "icon": None,
				  "marked": False,
				  "data": None,
				}]

		self.merge_messages(account["name"], new_messages)

		return True
		
	def can_popup(self):
		return False
		
	def start (self):
		self.hasNotified = False
		GIcon.GIcon.start(self)

	def __init__(self):
		self.prev_last_unread = 0
		self.prev_unread = 0
		self.total_unread = 0
		self.hasNotified = False
		
		self.config = "greader"
		self.last_unread = long(0)
		
		GIcon.GIcon.__init__(self)
	
