# -*- coding: utf-8 -*-

import googsystray
import gtk
import json
import copy
import time
import os

import GIcon
import GMain

class GWave(GIcon.GIcon):
	def draw_config(self, vbox):
		GIcon.GIcon.draw_config(self, vbox)

		check = gtk.CheckButton(_("Only notify once on unread messages."))
		check.set_tooltip_text(_("Googsystray will notify only once when an unread wave is detected."))
		check.connect("toggled", self.check_toggled, "notify_once")
		self.sens_widgets.append(check)
		check.set_data("make_sensitive", 1)
		check.set_active(GMain._GConf.backupconfig[self.config]["notify_once"])
		vbox.pack_start(check, False, False, 5)

	def label_clicked(self, widget, event, message=None):
		url = message["data"]["msg_id"]
		c = url.find('+')
		
		locstr = url[c+1:]
		locstr = locstr.replace("+", "%252B")
		locstr = locstr.replace("@", "%2540")
		
		url = "https://wave.google.com/wave/#restored:wave:" + url[:c] + "%252B" + locstr
		
		self.browser_open(url)
		
		self.thread_lock.acquire()
		for i in self.messages_unread:
			if i["id"] == message["id"]:
				i["marked"] = True
				break
			
		self.thread_lock.release()
		
		
	def draw_message(self, message):
		mvbox = self.popup_window.vbox
		
		vbox = gtk.VBox()
		mvbox.pack_start(vbox, False, False, 0)
		vbox.set_data("msg_id", message["id"])
		self.popup_messages.append(vbox)

		wave = message["data"]
		sep = gtk.HSeparator()
		vbox.pack_start(sep, False, False, 4)


		title_button = GIcon.link_label()
		title_button.set_markup("<span color=\"#0000FF\" size=\"large\"><u>" + wave["title"] + "</u></span>")
		title_button.connect("button_press_event", self.label_clicked, copy.deepcopy(message))
		
		
		unread_label = gtk.Label()
		unread_label.set_markup("<b>(" + str(wave["unread"]) + "/" + str(wave["total"]) + ")</b>")
		time_label = gtk.Label()
		timestamp = time.ctime(wave["timestamp"])
		time_label.set_markup("  <span color=\"#404040\">" + timestamp + "</span>")
		hbox1 = gtk.HBox()
		hbox1.pack_start(title_button, False, False, 5)
		hbox1.pack_start(unread_label, False, False, 5)
		if self.num_used() > 1:
			acct_label = gtk.Label()
			acct_label.set_markup("  <span color=\"#404040\">" + message["account"] + "</span>")
			hbox1.pack_start(acct_label, False, False, 5)
		hbox1.pack_end(time_label, False, False, 5)
		vbox.pack_start(hbox1, False, False, 3)
		
		hbox2 = gtk.HBox()
		vbox.pack_start(hbox2, False, False, 3)
		
		
		vbox1 = gtk.VBox()
		vbox2 = gtk.VBox()
		c = 0
		for i in wave["emails"]:
			if c >= 3:
				break
			image = self.get_portrait_address(i)
			if image is not None:
				vbox1.pack_start(image, False, False, 3)
				c += 1
		hbox2.pack_start(vbox1, False, False, 3)
		hbox2.pack_start(vbox2, False, False, 3)

		for i in wave["msgs"]:
			text = i["msg"]
			if not i["whole"]:
				text += "..."
			label = gtk.Label(text)
			label.set_line_wrap(True)
			hbox = gtk.HBox()
			hbox.pack_start(label, False, False, 3)
			vbox2.pack_start(hbox, False, False, 1)
	
		
	def get_portrait_address(self, address, size=32):
		if address == self.my_username:
			return self.get_portrait_fname(self.my_portrait, size)
		for i in self.contacts:
			if i["email"] == address:
				return self.get_portrait_fname(i["photo"], size)
		return self.get_portrait_fname("None", size)	
		
	def get_portrait_fname(self, portrait, size=32):
		if portrait == "None":
			pfname = os.path.join(googsystray.BASE_DIR, "icons", "Unknown-Portrait.png")
		elif portrait.startswith("http://"):
			pfname = os.path.join(googsystray.temp_dir, os.path.normpath(portrait[7:]))
		elif portrait.startswith("https://"):
			pfname = os.path.join(googsystray.temp_dir, os.path.normpath(portrait[8:]))
		else:
			pfname = os.path.join(googsystray.temp_dir, os.path.normpath("s2" + portrait))
			
		
		if not os.path.exists(pfname):
			print "FIXME USE A DEFAULT CRAPPY PORTRAIT HERE"
			# We shouldn't ever really hit here,
			# but for some reason nt and linux have different
			# needs on whether the main lock is set.
			if os.name == "nt":
	                        gtk.gdk.threads_enter()
			self.download_portrait(account, portrait)
			if os.name == "nt":
				gtk.gdk.threads_leave()
			
		pb = gtk.gdk.pixbuf_new_from_file(pfname)
		image = None
		
		if pb.get_width() == size and pb.get_height() == size:
			image = gtk.image_new_from_pixbuf(pb)
		elif pb.get_width() == pb.get_height():
			pb2 = pb.scale_simple(size, size, gtk.gdk.INTERP_HYPER)
			image = gtk.image_new_from_pixbuf(pb2)
		elif pb.get_height() == size or pb.get_width() == size:
			pb2 = gtk.gdk.Pixbuf(pb.get_colorspace(), pb.get_has_alpha(), pb.get_bits_per_sample(), size, size)
			pb2.fill(0xffffffff)
			pb.copy_area(0, 0, pb.get_width(), pb.get_height(), pb2, 0, 0)
			image = gtk.image_new_from_pixbuf(pb2)
			
		return image

	def check_jdata(self, account, data):
		waves = data["p"]["1"]
		new_messages = []
		
		for i in waves:
			msgs = []
			emails = []
			try:
				if i["7"] == 0:
					continue
			except:
				print "Error reading wave data!"
				print i
				continue
			t = 4294967295L
			t *= (i["8"][1] + 1)
			t += i["8"][0]
			t /= 1000

			
			for j in i["5"]:
				emails.append(j)
			
			for j in i["10"]:
				
				if j["5"]["2"] == 0:
					continue
				
				msgs.append({"msg":j["1"],
				             "whole":j["3"]})



			# Only ID here... we don't want to ping each
			# time a wave is updated, that could get insane.
			# Only on first new message.
			new_messages.append(
				{ "id": i["1"],
				  "sub_id": "wave",
				  "account": account["name"],
				  "count": 1,
				  "title": i["9"]["1"], # and account, maybe?
				  "summary": "Insert Wave Summary Here Fixme",
				  "icon": None,
				  "marked": False,
				  "data": {"title":i["9"]["1"],
					      "unread":i["7"],
					      "total":i["6"],
					      "timestamp":t,
					      "msg_id":i["1"],
					      "msgs":msgs,
					      "emails":emails,
				     }})
			     
			
			     
			    
					  
		
		self.merge_messages(account["name"], new_messages)
		
			

	def check_contacts(self, account, data):
		
		for i in data["p"]["2"]:
			#print i
			email=i["3"]["8"] # I think always email addy
			photo=i["3"].get("2")
			if photo == None:
				photo = i["6"].get("2")
			if photo == None:
				photo = "None"
					
			realname=i["3"].get("1")
			if realname == None:
				realname = i["6"].get("1")


			found = False
			for j in self.contacts:
				if j["email"] == email:
					j["photo"] = photo
					j["realname"] = realname
					found = True
					break
			
			if not found:
				self.contacts.append({"email":email,"photo":photo,"realname":realname})
			
			if photo != "None":
				self.download_portrait(account, photo)
			

		
	def download_portrait(self, account, portrait):
		if portrait.startswith("http://"):
			pfname = os.path.join(googsystray.temp_dir, os.path.normpath(portrait[7:]))
			purl = portrait
		elif portrait.startswith("https://"):
			pfname = os.path.join(googsystray.temp_dir, os.path.normpath(portrait[8:]))
			purl = portrait
		else:
			pfname = os.path.join(googsystray.temp_dir, os.path.normpath("s2" + portrait))
			purl = "https://wave.google.com/wave/c" + portrait
		
		if not os.path.exists(pfname):
			data = self.urlopen2(account, purl)

			if data == None:
				return False
				
			(pdir, pfile) = os.path.split(pfname)

			if not os.path.exists(pdir):
				os.makedirs(pdir)

			pf = open(pfname, "wb")
			pf.write(data)
			pf.close()
			

	def icon_check_data(self, account):
		data = self.urlopen2(account, 'https://wave.google.com/wave/')
		if data == None:
			return False

		if self.my_portrait is None:
			c_start = data.find("userProfile:")
			id_start = data.find("id:'", c_start)
			id_end = data.find("'", id_start+4)
			username_start = data.find("username:'", c_start)
			username_end = data.find("'", username_start+10)
			username = data[username_start+10:username_end]
			self.my_username = username
			id = data[id_start+4:id_end]
			
			print account["name"] + " " +  username
			
			url = "https://wave.google.com/wave/wfe/fetch/googlewave.com!u+profile+%s/%s?v=3" % (username, id)
			profiledata = self.urlopen2(account, url)
			jdata = json.loads(profiledata[5:])
			# fixme... you gotta be kidding me :)
			# I certainly hope they never change this :)
			# Or maybe hoping somebody really deconstructs what all this
			# crap means... 
			for i in jdata["1"][1]["1"]["2"][0]["16"]["2"]:
				if i.get("4"):
					if i["4"]["1"] == "profile":
						for j in i["4"]["2"]:
							if j["1"] == "avatar-url":
								self.my_portrait=j["2"]
			
			if self.my_portrait is not None:
				self.download_portrait(account, self.my_portrait)
			else:
				self.my_portrait = "None"
			


		c_end = 0
		c_start = data.find("var json = ", c_end)
		
		

		while c_start != -1:
			c_end = data.find("}};\n", c_start)
			jdata = json.loads(data[c_start+11:c_end+2])
			if jdata["t"] == 2602:
				self.check_jdata(account, jdata)
			if jdata["t"] == 2033:
				self.check_contacts(account, jdata)
		
			c_start = data.find("var json =", c_end)
		
		return True
		
	def start (self):
		self.has_notified = False
		GIcon.GIcon.start(self)

	def __init__(self):
		# Yes.  Only one set of contacts for all possible accounts --
		# Why?  It doesn't matter :)		
		self.contacts = []
		self.has_notified = False
		self.my_portrait = None
		self.my_username = None
		
		self.config = "gwave"
		
		GIcon.GIcon.__init__(self)
	
