# -*- coding: utf-8 -*-

import os
import gettext
import locale
import time

#########################################################
# Config
#########################################################

VERSION_MAJOR = 1
VERSION_MINOR = 2
VERSION_REV   = 0
try:
	import googsystray_build
	VERSION_BUILD = googsystray_build.EXTRA
except ImportError:
	VERSION_BUILD = ""
VERSION_STRING = str(VERSION_MAJOR) + "." + str(VERSION_MINOR) + "." + str(VERSION_REV) + str(VERSION_BUILD)

DEFAULT_TIMEOUT = 180000

#class ConfigWindow:

ONCLICK_NOACTION = -1
ONCLICK_OPEN = 0
ONCLICK_NEW_WINDOW = 1
ONCLICK_NEW_TAB = 2
ONCLICK_OTHER = 3


if (os.name == "posix"):
	if os.getenv("USER") is None:
		# This case apparently appears on gentoo emerge installs.
		# I don't believe any of these variables will actually
		# be used during the install
		temp_dir = "/tmp" 
		config_dir = "/tmp"
	else:
		temp_dir = "/tmp/googsystray-" + os.getenv("USER") + "/"
		if not os.path.exists(temp_dir):
			os.makedirs(temp_dir)

		config_dir = os.getenv("HOME")
	config_name = ".googsystrayrc"
	
elif (os.name == "nt"):
	config_dir = os.getenv("APPDATA")
	print config_dir
	temp_dir = os.getenv("TEMP")
	config_name = "googsystray.ini"
else:
	print "Error: Unable to locate temp directory for " + os.name




BASE_DIR = "/tmp"

def debug(debug_str):
	print time.asctime() + ": " + str(debug_str)
	return

def error(error_str):
	return

def set_basedir(bdir):
	global BASE_DIR
	BASE_DIR = bdir
	locale.setlocale(locale.LC_ALL, "")
	(lang_str, cp) = locale.getlocale()
	if os.name == "nt":
		# Oh, damn you windows.  We'll need to add here for any
		# language we wish to support.
		(language, sep, country) = lang_str.partition("_")
		new_language = "en" # We'll just default to English.
		new_country = "US" # And United States.

		# This set is important for translations
		if language == "Chinese (Simplified)":
			new_language = "zh"
			new_country = "CN"
		elif language == "Chinese (Traditional)":
			new_language = "zh"
			new_country = "TW"
		elif language == "Dutch":
			new_language = "nl"
		elif language == "English":
			new_language = "en"
		elif language == "French":
			new_language = "fr"
		elif language == "German":
			new_language = "de"
		elif language == "Gujarati":
			new_language = "gu"
		elif language == "Hindi":
			new_language = "hi"
		elif language == "Hungarian":
			new_language = "hu"
		elif language == "Italian":
			new_language = "it"
		elif language == "Japanese":
			new_language = "ja"
		elif language == "Russian":
			new_language = "ru"
		elif language == "Spanish":
			new_language = "es"
		elif language == "Polish":
			new_language = "pl"
		elif language == "Portuguese":
			new_language = "pt"
		else:
			print "Unrecognized lang: " + language

		# This set is important for dates/times/etc
		if country == "Australia":
			new_country = "AU"
		elif country == "Canada":
			new_country = "CA"
		elif country == "France":
			new_country = "FR"
		elif country == "Germany":
			new_country = "DE"
		elif country == "Japan":
			new_country = "JA" # ???
		elif country == "Russia":
			new_country = "RU"
		elif country == "United Kingson":
			new_country = "UK"
		elif country == "United States":
			new_country = "US"
		else:
			print "Unrecognized country: " + country


		if new_country is None:
			os.environ['LANG'] = new_language
		else:
			os.environ['LANG'] = new_language + "_" + new_country

	if gettext.find("googsystray", os.path.join(BASE_DIR, "po")) == None:
		# This should only get called on a Linux install, where the
		# .mo files are installed to a system locale directory.
		# However, we will also get here on a Windows system where
		# we do not have proper translations (Or English)
		gettext.install("googsystray", names=['ngettext'])
	else:
		gettext.install("googsystray", os.path.join(BASE_DIR, "po"), names=['ngettext'])
		

def get_basedir():
	return os.path.dirname(__file__)
	
_installed = False
def get_installed():
	global _installed
	return _installed

def set_installed(i):
	global _installed
	_installed = i
